
/******************************************************************************
 *
 *  CFingKeyWndTask.c
 *
 *	Fingerspelling key window task classes for the SignWriter key editor.
 *
 *  Copyright 1996 The Center for Sutton Movement Writing. All rights reserved.
 *
 ******************************************************************************/
 


#include "CFingKeyWndTask.h"
#include "CFingKeyWndPane.h"
#include "CSymWindow.h"
#include "KeyEditConsts.h"
#include "Global.h"
#include "Keys.h"
#include "Symbols.h"
#include "SymCode.h"
#include "TBUtilities.h"




/*** Global Variables ***/


extern void	  		*gClipRec;				/* Clipboard */
extern void	  		*gScratchRec;
extern ObjType		 gScratchType;


extern CSymWindow	*gSymWind;				/* Symbol window access */





/********************************  Clear  **********************************/


/***
 * IFingKeyWndClear
 *
 ***/
 
void CFingKeyWndClear::IFingKeyWndClear (short nameIndex, CFingKeyWndPane *pane, 
 														  			Byte keyCode)		
{
	CTask::ITask(nameIndex);
	
	itsPane = pane;
	itsKeyCode  = keyCode;
	
}	



/***
 * Do (override)
 *
 * Clear the selected symbol or key variation box.
 * Save original in the global scratch record.
 *
 ***/

void CFingKeyWndClear::Do()

{
	Box	 	  rec;
	ObjType   selType;
	short	  selInx;
	
	
	GetFingerKey(itsKeyCode, &rec);
	
	GetSelectType(rec, &selType);
	GetSelectSym(rec, &selInx);
	
	
	if (gScratchRec != NULL)
		DisposKey(gScratchType, gScratchRec);
	
	CopyFingerKey(rec, (Box *) &gScratchRec);
	gScratchType = selType;


	if (selType == box)
		ClearFingerKey(rec);				
 	else 
 		DeleteSym(rec, selInx);
 		
	
	SetSelectType(rec, none);

	itsPane->UpdateKey();	

}
		


/***
 * Undo (override)
 *
 *	Exchange the current selection
 *  with the global scratch record.
 *
 ***/

void CFingKeyWndClear::Undo()

{
	Box	 rec;
	
	
	GetFingerKey(itsKeyCode, &rec);
	SetFingerKey(itsKeyCode, (Box) gScratchRec);
	gScratchRec = rec;
					
	itsPane->UpdateKey();	
	
}




/********************************  Paste  **********************************/


/***
 * IFingKeyWndPaste
 *
 ***/
 
void CFingKeyWndPaste::IFingKeyWndPaste (CFingKeyWndPane *pane, 
										 			Byte keyCode)		
{
	CTask::ITask(iPASTE);
	
	itsPane = pane;
	itsKeyCode  = keyCode;
	
}	
	


/***
 * Do (override)
 *
 * Paste the clipboard item into the current selection.
 * Save original in the global scratch record.
 *
 * If the clipboard item is a symbol, add it to the 
 * selected box (if there is room).
 * If the clipboard item is a box, replace the
 * selected box with the clipboard item.
 *
 ***/

void CFingKeyWndPaste::Do()

{
	Box	 	  rec;
	ObjType   selType, clipType;
	short	  selInx, clipInx;
	short	  symCount, junk;
	
	
	GetFingerKey(itsKeyCode, &rec);
	
	GetSelectType(rec, &selType);
	GetSelectSym(rec, &selInx);
				
	GetSelectType((Box) gClipRec, &clipType);
	GetSelectSym((Box) gClipRec, &clipInx); 
	
	
	if (gScratchRec != NULL)
		DisposKey(gScratchType, gScratchRec);
				
				
	if (clipType == sym) {								// clipboard holds symbol...
	
		
		CopyFingerKey(rec, (Box *) &gScratchRec);
		gScratchType = selType;


		switch (selType) {
		
			case none:
			
				GetSymCount(rec, &symCount);
				
				if (symCount >= KEY_SYMS) {					// key is full 
				
					PositionDialog('ALRT', ALRTkeyFull);	
					junk = Alert(ALRTkeyFull, NULL);	
					
					return;
				}	
				break;
					
			case box:										// clear current selection
			
				ClearFingerKey(rec);
				break;
						
			case sym:
				
				DeleteSym(rec, selInx);
				break;				
		}
					
		CopySym((Box) gClipRec, clipInx, rec);				// paste symbol

	
 	} else {    										// clipboard holds box...
 	
 	
 		gScratchRec  = rec;									// move current to scratch
 		gScratchType = selType;
	
		CopyFingerKey((Box) gClipRec, &rec);				// copy clip to current
 		SetSelectType(rec, none);
 			
 		SetFingerKey(itsKeyCode, rec);			
 		
	}
			
	itsPane->UpdateKey();			

}
	
	

/***
 * Undo (override)
 *
 *	Exchange the current selection
 *  with the global scratch record.
 *
 ***/

void CFingKeyWndPaste::Undo()

{
	Box	 rec;
	
	
	GetFingerKey(itsKeyCode, &rec);
	SetFingerKey(itsKeyCode, (Box) gScratchRec);
	gScratchRec = rec;
					
	itsPane->UpdateKey();	
	
}




/*********************************  Add  ***********************************/


/***
 * IFingKeyWndAdd
 *
 ***/
 
void CFingKeyWndAdd::IFingKeyWndAdd (CFingKeyWndPane *pane, 
									 			Byte keyCode)	
{
	CTask::ITask(iADD);
	
	itsPane = pane;
	itsKeyCode  = keyCode;
	
}	
	


/***
 * Do (override)
 *
 * Add symbol selected in sym window to the 
 * current selection.  Save original in the
 * global scratch record.  Center symbol
 * in the key box -- leave it selected.
 *
 ***/

void CFingKeyWndAdd::Do()

{
	Box	 	  rec;
	ObjType   selType;
	short	  selInx;
	short	  symNum, symCount, junk;
	short	  symXPos, symYPos;
	LongPt	  symFrame, unused;
	Byte	  *symBase;	
	
	
	GetFingerKey(itsKeyCode, &rec);
	
	GetSelectType(rec, &selType);
	GetSelectSym(rec, &selInx);
	
	if (gScratchRec != NULL)
		DisposKey(gScratchType, gScratchRec);
		
	CopyFingerKey(rec, (Box *) &gScratchRec);
	gScratchType = selType;


	switch (selType) {
	
		case none:
		
			GetSymCount(rec, &symCount);
			
			if (symCount >= KEY_SYMS) {					// key is full 
			
				PositionDialog('ALRT', ALRTkeyFull);	
				junk = Alert(ALRTkeyFull, NULL);	
				
				return;
			}	
			break;
				
		case box:										// clear current selection
		
			ClearFingerKey(rec);
			break;
					
		case sym:
			
			DeleteSym(rec, selInx);
			break;				
	}
			
			
	GetSymCount(rec, &symCount);						// add symbol...					
	GetSymBase(rec, symCount, &symBase);
	
 	SetSelectType(rec, sym);
 	SetSelectSym(rec, symCount);
 	SetSymCount(rec, symCount + 1);
	
 	
	NewSymCode(symBase);								// build sym code
	
	symNum = gSymWind->SelectedSymbol();
	SetSymNum(symBase, symNum);

	GetCoords(symNum, 0, 0, 0, 0, &symFrame, &unused, FALSE); 
	
	symXPos = (FKPANE_WIDTH  - symFrame.h) / 2;	
	symYPos = (FKPANE_HEIGHT - symFrame.v) / 2;	
	
	SetSymXPos(symBase, symXPos);	
	SetSymYPos(symBase, symYPos);		
	
	
	SetDescender(rec, symYPos + symFrame.v);			// update descender
		
 
	itsPane->UpdateKey();			

}
	


/***
 * Undo (override)
 *
  *	Exchange the current selection
 *  with the global scratch record.
 *
 ***/

void CFingKeyWndAdd::Undo()

{
	Box	 rec;
	
	
	GetFingerKey(itsKeyCode, &rec);
	SetFingerKey(itsKeyCode, (Box) gScratchRec);
	gScratchRec = rec;
					
	itsPane->UpdateKey();	
	
}




/*******************************  Symbol Drag  ********************************/


/***
 * IFingKeyWndDrag
 *
 * 	Initialize mouse drag class.  Set class
 *  variables itsPane, itsBase (base of symbol code), 
 *  and itsFrame (size of symbol being dragged).
 *
 ***/
  
void CFingKeyWndDrag::IFingKeyWndDrag (CFingKeyWndPane *pane, Box key, short symInx)
 
{
 	
 	short	code, var, fill, rot, flop, junk;
 	LongPt  notUsed;
 	
 	
 	itsPane = pane;
	GetSymBase(key, symInx, &itsBase);
	
	GetSymInfo(itsBase, &code, &var, &fill, &rot, &flop, &junk, &junk, &junk);
	GetCoords(code, var, fill, rot, flop, &itsFrame, &notUsed, FALSE); 	
	
}



/***
 * BeginTracking (override default method)
 *
 * Prepare for tracking by initially
 * drawing the selected symbol.
 *
 ***/
 
void CFingKeyWndDrag::BeginTracking (LongPt *startPt)

{
	itsPane->ToggleSelect();								// display sym
}



/***
 * KeepTracking (override default method)
 *
 * Move symbol according to 
 * last mouse movement.
 *
 ***/
 
void CFingKeyWndDrag::KeepTracking (LongPt *currPt, LongPt *prevPt, LongPt *startPt)

{
	short  top, left, right, bottom;
		
		
	if (EqualLongPt(currPt, prevPt))
		return;
	

	itsPane->ToggleSelect();							// erase sym
	
	
	GetSymXPos(itsBase, &left);							// get current sym position
	GetSymYPos(itsBase, &top);								 
							
							
	left += (currPt->h - prevPt->h);					// offset by mouse movement			
	top  += (currPt->v - prevPt->v);				
		
	right  = left + itsFrame.h;
	bottom = top  + itsFrame.v;
	
	
	if (left < 0) left = 0;								// check bounds
	if (top  < 0) top  = 0;
	
	if (right  >= SKCELL_WIDTH)  left = SKCELL_WIDTH  - itsFrame.h - 1;
	if (bottom >= SKCELL_HEIGHT) top  = SKCELL_HEIGHT - itsFrame.v - 1;


	SetSymXPos(itsBase, left);							// update sym position
	SetSymYPos(itsBase, top);	
					
					
	itsPane->ToggleSelect();							// redraw updated sym
				 								
}




/****************************  Descender Drag  *****************************/


/***
 * IFingKeyWndDesc
 *
 * 	Initialize descender drag class.
 *  Set class variable itsKeyRec.
 *
 ***/
  
void CFingKeyWndDesc::IFingKeyWndDesc (CFingKeyWndPane *pane, Box key)
 
{ 	
 	itsPane = pane;
 	itsKeyRec = key;	
}



/***
 * BeginTracking (override default method)
 *
 * Prepare for tracking by initially
 * drawing the descender line.
 *
 ***/
 
void CFingKeyWndDesc::BeginTracking (LongPt *startPt)

{
	//itsPane->ToggleDescender();								// erase descender
}



/***
 * KeepTracking (override default method)
 *
 * Move descender line according
 * to last mouse movement.
 *
 ***/
 
void CFingKeyWndDesc::KeepTracking (LongPt *currPt, LongPt *prevPt, LongPt *startPt)

{
	short 	descend;
	
		
	if (EqualLongPt(currPt, prevPt))
		return;
		

	itsPane->ToggleDescender();							// erase descender
	
	
	GetDescender(itsKeyRec, &descend);					// offset by mouse movement									
	descend += (currPt->v - prevPt->v);	
														
														// check bounds
	if (descend < 0) descend = 0;							
	if (descend >= FKPANE_HEIGHT)
		descend = FKPANE_HEIGHT - 1;


	SetDescender(itsKeyRec, descend);					// update descender					
					
	itsPane->ToggleDescender();							// redraw 
				 								
}

