/******************************************************************************
 CToolShapes.c

							The ToolShapes Class
	
	This file actually contains methods for eight different classes,
	corresponding to the tool for drawing geometric shapes:
		rectangles, rounded rectangles, and ovals (hollow and filled).
		and a line. The CToolShapes Class is the superclass of all
		these classes.
		
	The different classes implement only one method, UseTool(), except
	for the line which has a couple other methods. The other methods
	are all inherited from the parent CToolShapes class.

	SUPERCLASS = CPaintTask
	
	Copyright  1989 Symantec Corporation. All rights reserved.

 ******************************************************************************/

#include <Global.h>
#include <TBUtilities.h>
#include "CToolShapes.h"

extern CBitMap		*gScratchPad;
extern Pattern		gPattern;			/* The current fill pattern			*/
extern short		gLineSize;

#define		SHIFTKEY_CODE	0x38		/* Hardware code for the shift key	*/
#define		iPAINTING		7			/* Index for Painting task name		*/


/**** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S ****/


/******************************************************************************
 ToolShapes

		Initialize a ToolShapes object
 ******************************************************************************/

void	CToolShapes::IToolShapes(
	CPaintPane	*aPaintPane,
	CBitMap		*aPainting)
{
	CPaintTask::IPaintTask(iPAINTING, aPaintPane, aPainting);
}


/******************************************************************************
 BeginTracking {OVERRIDE}

		Mouse down tracking is about to begin at the specified starting
		LongPt. Set the pen size.
 ******************************************************************************/

void	CToolShapes::BeginTracking(
	LongPt		*startPt)
{
	gScratchPad->BeginDrawing();
	PenSize(gLineSize, gLineSize);
	gScratchPad->EndDrawing();
}


/******************************************************************************
 KeepTracking {OVERRIDE}

		Continuous mouse tracking while the mouse button is down. Drawing
		is performed in an offscreen scratch pad and then copied to the
		screen. The offscreen painting image does not change during this
		dynamic drawing.
 ******************************************************************************/

void	CToolShapes::KeepTracking(
	LongPt		*currPt,
	LongPt		*prevPt,
	LongPt		*startPt)
{
	LongRect		changedRect;
	LongRect		currRect;
	
	TweakPoints(currPt, prevPt, startPt);
	if (itsPaintPane->AutoScroll( currPt) || !EqualLongPt( currPt,  prevPt)) {
		itsPaintPane->GetFrame(&currRect);
		PinInRect(&currRect, currPt);
										/* Find area of painting that has	*/
										/*   has changed					*/
		CalcChangedRect( currPt, prevPt, startPt, &changedRect);
		
										/* Copy changed area to scratch pad	*/
		LCopyBits(itsPainting->macBitMap, gScratchPad->macBitMap,
				 &changedRect, &changedRect, srcCopy, NULL);
				 
		gScratchPad->BeginDrawing();	/* Do drawing offscreen on the		*/
										/*   scratchpad						*/
		Pt2LongRect( currPt,  startPt, &currRect);
		UseTool(&currRect, currPt, startPt);
		gScratchPad->EndDrawing();
		
										/* Copy from scratch pad onto the	*/
										/*   screen							*/
		gScratchPad->CopyFrom(&changedRect, &changedRect, NULL);
	}
}


/******************************************************************************
 EndTracking {OVERRIDE}

		Mouse down tracking has ended because the user has released the
		mouse button. Its now time to set up the undo buffer and change
		the actual offscreen painting image.
 ******************************************************************************/

void	CToolShapes::EndTracking(
	LongPt		*currPt,
	LongPt		*prevPt,
	LongPt		*startPt)
{
	LongRect		currRect;

									/* Track for the last time				*/
	KeepTracking(currPt, prevPt, startPt);

	CalcChangedRect( currPt,  currPt,  startPt, &currRect);
	
	SaveForUndo(&currRect);
	
									/* Copy drawing area to scratch pad		*/
	LCopyBits(itsPainting->macBitMap, gScratchPad->macBitMap,
			 &currRect, &currRect, srcCopy, NULL);
	gScratchPad->BeginDrawing();	/* Draw new object on top of existing	*/
									/*   drawing on the scratchpad			*/
	UseTool(&currRect, currPt, startPt);
	gScratchPad->EndDrawing();

									/* Copy drawing area from scratchpad	*/
									/*   into the painting image			*/
	LCopyBits(gScratchPad->macBitMap, itsPainting->macBitMap,
			 &currRect, &currRect, srcCopy, NULL);
}


/******************************************************************************
 TweakPoints

		Adjust locations of the construction points. Contrains
		currPt so that the rectangle formed by startPt & currPt is
		a square if the shift key was down at the time of the
		original mouse click.
 ******************************************************************************/

void	CToolShapes::TweakPoints(
	LongPt		*currPt,
	LongPt		*prevPt,
	LongPt		*startPt)
{
	short 		length;
	
	if (KeyIsDown(SHIFTKEY_CODE)) {
		length = Min(Abs(startPt->h - currPt->h), Abs(startPt->v - currPt->v));
		if (currPt->h > startPt->h) {
			currPt->h = startPt->h + length;
		} else {
			currPt->h = startPt->h - length;
		}
		if (currPt->v > startPt->v) {
			currPt->v = startPt->v + length;
		} else {
			currPt->v = startPt->v - length;
		}
	}
}


/******************************************************************************
 CalcChangedRect

		Calculate bounding rectangle of area affected by drawing a shape
		at the specified locations
 ******************************************************************************/

void	CToolShapes::CalcChangedRect(
	LongPt		*currPt,
	LongPt		*prevPt,
	LongPt		*startPt,
	LongRect		*changedRect)
{
	LongRect		currRect;
	LongRect		prevRect;

	Pt2LongRect(currPt, startPt, &currRect);
	Pt2LongRect(prevPt, startPt, &prevRect);
	UnionLongRect(&currRect, &prevRect, changedRect);
}


/******************************************************************************
 UseTool
 ******************************************************************************/

void	CToolShapes::UseTool(
	LongRect		*currRect,
	LongPt		*currPt,
	LongPt		*startPt)
{
}									/* Null Method							*/


/**** UseTool Methods for Different Shape Tools ****/


/******************************************************************************
 UseTool {Rectangle}
 ******************************************************************************/

void	CToolRect::UseTool(
	LongRect		*currRect,
	LongPt		*currPt,
	LongPt		*startPt)
{
	LFrameRect(currRect);
}


/******************************************************************************
 UseTool {Filled Rectangle}
 ******************************************************************************/

void	CToolFillRect::UseTool(
	LongRect		*currRect,
	LongPt		*currPt,
	LongPt		*startPt)
{
	LFillRect(currRect, gPattern);
	LFrameRect(currRect);
}


/******************************************************************************
 UseTool {Rounded Rectangle}
 ******************************************************************************/

void	CToolRRect::UseTool(
	LongRect		*currRect,
	LongPt		*currPt,
	LongPt		*startPt)
{
	LFrameRoundRect(currRect, 15, 15);
}


/******************************************************************************
 UseTool {Filled Rounded Rectangle}
 ******************************************************************************/

void	CToolFillRRect::UseTool(
	LongRect		*currRect,
	LongPt		*currPt,
	LongPt		*startPt)
{
	LFillRoundRect(currRect, 15, 15, gPattern);
	LFrameRoundRect(currRect, 15, 15);
}


/******************************************************************************
 UseTool {Oval}
 ******************************************************************************/

void	CToolOval::UseTool(
	LongRect		*currRect,
	LongPt		*currPt,
	LongPt		*startPt)
{
	LFrameOval(currRect);
}


/******************************************************************************
 UseTool {Filled Oval}
 ******************************************************************************/

void	CToolFillOval::UseTool(
	LongRect		*currRect,
	LongPt		*currPt,
	LongPt		*startPt)
{
	LFillOval(currRect, gPattern);
	LFrameOval(currRect);
}


/******************************************************************************
 UseTool {Line}
 ******************************************************************************/

void	CToolLine::UseTool(
	LongRect		*currRect,
	LongPt		*currPt,
	LongPt		*startPt)
{
	LMoveTo(startPt->h, startPt->v);
	LLineTo(currPt->h, currPt->v);
}


/******************************************************************************
 TweakPoints {Line}

		Adjust locations of the construction points.
 ******************************************************************************/

void	CToolLine::TweakPoints(
	LongPt		*currPt,
	LongPt		*prevPt,
	LongPt		*startPt)
{
}


/******************************************************************************
 CalcChangedRect {Line}

		Calculate bounding rectangle of area affected by drawing a line
		at the specified locations
 ******************************************************************************/

void	CToolLine::CalcChangedRect(
	LongPt		*currPt,
	LongPt		*prevPt,
	LongPt		*startPt,
	LongRect	*changedRect)
{
	LongPt			a, curr, prev;
	LongRect		currRect;
	LongRect		prevRect;
	
	curr = *currPt;
	prev = *prevPt;
	
	a = *startPt;						/* We have to adjust the points a	*/
	if (curr.h >= startPt->h) {			/*   bit to compensate for the pen,	*/
		curr.h += gLineSize;			/*   which hangs below and to the	*/
	} else {							/*   right its QuickDraw coordinate	*/
		a.h += gLineSize;
	}
	
	if (curr.v >= startPt->v) {
		curr.v += gLineSize;
	} else {
		a.v += gLineSize;
	}
	
	Pt2LongRect( &a, &curr, &currRect);
	
	a = *startPt;
	if (prev.h >= startPt->h) {
		prev.h += gLineSize;
	} else {
		a.h += gLineSize;
	}
	
	if (prev.v >= startPt->v) {
		prev.v += gLineSize;
	} else {
		a.v += gLineSize;
	}
	
	Pt2LongRect( &a, &prev, &prevRect);
	
	UnionLongRect(&currRect, &prevRect, changedRect);
}
