/******************************************************************************
 OCluster.c

							The Cluster Class
							
		An unordered group of objects.
	
	SUPERCLASS = CCollection
	
	Copyright  1989 Symantec Corporation. All rights reserved.
	

 ******************************************************************************/

#include "Global.h"
#include "OCluster.h"


/**** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S ****/
	

/******************************************************************************
 ICluster

		Initialize a Cluster object
 ******************************************************************************/

void	OCluster::ICluster()
{
	CCollection::ICollection();			/* Initialize superclass			*/
	
	blockSize = 3;						
	slots = 0;
	items = (LongHandle) NewHandle(0L);
	FailNIL( items);
}
	

/******************************************************************************
 Dispose {OVERRIDE}

		Dispose of a Cluster. However, do not dispose of the items contained
		in the Cluster.
 ******************************************************************************/

void	OCluster::Dispose()
{
	DisposHandle( (Handle) items);		/* Free handle to items				*/
	inherited::Dispose();				/* Pass message on to superclass	*/
}
	

/******************************************************************************
 DisposeAll

		Dispose of all items contained in a Cluster and the Cluster itself.
 ******************************************************************************/

void	OCluster::DisposeAll()
{
	register long	i;					/* Index into Cluster				*/
	CObject			*itemToPurge;
	
	for (i = 0; i < numItems; i++) {	/* Run through items in the Cluster	*/
										/*   Send Dispose msg to each item	*/
		itemToPurge = (CObject*) (*items)[i];
		(*items)[i] = 0L;
		itemToPurge->Dispose();
	}
		
	this->Dispose();					/* Dispose of the Cluster itself	*/
}
	

/******************************************************************************
 DisposeItems

		Dispose of all items contained in a Cluster, but NOT the
		Cluster itself
 ******************************************************************************/

void	OCluster::DisposeItems()
{
	register long	i;					/* Index into Cluster				*/
	CObject			*itemToPurge;
	
	for (i = 0; i < numItems; i++) {	/* Run through items in the Cluster	*/
										/*   Send Dispose msg to each item	*/
		itemToPurge = (CObject*) (*items)[i];
		(*items)[i] = 0L;				/* Clear reference to disposed item	*/
		itemToPurge->Dispose();
	}
	
	numItems = 0;						/* There are now no items			*/
	if (slots >  blockSize) {			/* There are too many free slots	*/
		slots = blockSize;				/* Reduce to one block of slots		*/
		SetHandleSize( (Handle) items, (long) (slots * SLOT_SIZE));
	}
	
}


/******************************************************************************
 SetBlockSize

		Set the blockSize which specifies the number of empty slots to
		allocate when more space is needed.
 ******************************************************************************/

void	OCluster::SetBlockSize(
	short		aBlockSize)				/* Specified blockSize				*/
{
	blockSize = aBlockSize;				/* Set instance variable			*/
}
	

/******************************************************************************
 Add

		Add a new object to the cluster
 ******************************************************************************/

void	OCluster::Add(
	CObject		*theObject)				/* Object to add					*/
{	
	if (numItems >= slots) {			/* Need to allocate more slots		*/
		MoreSlots();
	}
										/* Store object reference			*/
	(*items)[numItems] = (long)theObject;
	numItems++;							/* Increment count of items			*/
}	


/******************************************************************************
 Remove

		Remove an object from the cluster
 ******************************************************************************/

void	OCluster::Remove(
	CObject		*theObject)				/* Object to remove					*/
{
	register long		i;				/* Offset of object in cluster		*/
	
	i = Offset(theObject);				/* Search items for this object		*/
	
	if (i != BAD_INDEX) {
		numItems--;						/* Decrement count of items			*/
										/* Place last item in the slot		*/
		(*items)[i] = (*items)[numItems];
										/*   occupied by this object		*/
		(*items)[numItems] = 0L;		/* Clear slot of former last item	*/
		
		if (slots > numItems + blockSize) {
										/* The number of free slots is		*/
										/*   greater than the blockSize.	*/
										/*   Reduce the size of the items	*/
										/*   handle.						*/
			slots -= blockSize;
			SetHandleSize( (Handle) items, (long) (slots * SLOT_SIZE));
		}
	}
}


/******************************************************************************
 Includes

		Check whether an object is in the cluster
 ******************************************************************************/

Boolean	OCluster::Includes(
	CObject		*theObject)				/* Object to check for inclusion	*/
{
										/* Object is in the cluster if the	*/
										/*   Offset is NOT BAD_INDEX		*/
	return(Offset(theObject) != BAD_INDEX);
}


/******************************************************************************
 DoForEach

		Call a specified function for each object in the cluster.
 ******************************************************************************/

void	OCluster::DoForEach(
	VoidFunc	theFunc)				/* Function pointer					*/
{
	register long	i;
	
	for (i = 0; i < numItems; i++) {
		(*theFunc)((*items)[i]);
	}
}


/******************************************************************************
 DoForEach1

		Call a specified function for each object in the cluster. The
		function to call has one parameter, which is a long.
 ******************************************************************************/

void	OCluster::DoForEach1(
	VoidFunc	theFunc,				/* Function pointer					*/
	long		param)					/* Parameter for the function		*/
{
	register long	i;
	
	for (i = 0; i < numItems; i++) {
		(*theFunc)((*items)[i], param);
	}
}


/******************************************************************************
 FindItem

		Find the first item in the cluster which successfully passes (returns
		TRUE) the test function. Return NULL if no item passes the test.
 ******************************************************************************/

CObject*	OCluster::FindItem(
	BooleanFunc		testFunc)			/* Ptr to test function				*/
{
	register long	i;					/* Index into list of items			*/
	
	for (i = 0; i < numItems; i++) {	/* Scan from beginning to end		*/
		if ( (*testFunc)((*items)[i]) )
										/* Success! Pass back the object	*/
			return((CObject*)(*items)[i]);
	}
	
	return(NULL);						/* Entire list has been scanned		*/
										/*   with no successes				*/
}


/******************************************************************************
 FindItem1

		Find the first item in the cluster which successfully passes (returns
		TRUE) the test function. Return NULL if no item passes the test.
 ******************************************************************************/

CObject*	OCluster::FindItem1(
	BooleanFunc		testFunc,			/* Ptr to test function				*/
	long			param)				/* Parameter for test function		*/
{
	register long	i;					/* Index into list of items			*/
	
	for (i = 0; i < numItems; i++) {	/* Scan from beginning to end		*/
		if ( (*testFunc)((*items)[i], param) )
										/* Success! Pass back the object	*/
			return((CObject*)(*items)[i]);
	}
	
	return(NULL);						/* Entire list has been scanned		*/
										/*   with no successes				*/
}


/******************************************************************************
 Offset

		Find offset of an object within the cluster. Offset ranges
		from 0 to numItems-1.
 ******************************************************************************/

long	OCluster::Offset(
	CObject		*theObject)				/* Object to search for				*/
{
	register long	i;					/* Running index into the cluster	*/
	
	for (i = 0; i < numItems; i++) {	/* Search entire cluster			*/
										/* Return offset if object is found	*/
		if ( (long)theObject == (*items)[i] )
			return(i);
	}
										/* Falling thru the loop means that	*/
	return(BAD_INDEX);					/*   the search failed				*/
}


/******************************************************************************
 MoreSlots

		Allocate more slots for the items in a cluster
 ******************************************************************************/

void	OCluster::MoreSlots()
{
	slots += blockSize;
	SetHandleSize( (Handle) items, (long)(slots * SLOT_SIZE));
	FailMemError();
}
