/******************************************************************************
 ODataFile.c

							The File Class
		
	Class for a disk-based file. Information can be read from or written to
	a File.
	
	SUPERCLASS = CObject
	
	Copyright  1989 Symantec Corporation. All rights reserved.

 ******************************************************************************/

#include "Global.h"
#include "ODataFile.h"
#include "TCLUtilities.h"


/**** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S ****/


/******************************************************************************
 IDataFile

		Initialize a Data File object
 ******************************************************************************/

void	ODataFile::IDataFile()
{
	OFile::IFile();

	refNum = 0;
}


/**** A C C E S S I N G   M E T H O D S ****/


/******************************************************************************
 SetLength

		Set the length, in bytes, of the data fork of a file. File
		must be already open.
 ******************************************************************************/

OSErr	ODataFile::SetLength(
	long		aLength)
{
	return( SetEOF(refNum, aLength) );
}


/******************************************************************************
 GetLength

		Retrieve the length in bytes of the data fork of a file. File
		must be already open.
 ******************************************************************************/

OSErr	ODataFile::GetLength(
	long		*theLength)
{
	*theLength = 0L;
	return( GetEOF(refNum, theLength) );
}


/******************************************************************************
 SetMark

		Specify the position of the mark which determines determines the
		starting point for subsequent read/write operations. Position
		is specified as a distance from a certain location:
			fsFromStart, fsFromLEOF, fsFromMark (the beginning, end,
			or current mark).
		Positive distances offset towards the end of the file, negative
		ones towards the beginning. File must already be open.
 ******************************************************************************/

OSErr	ODataFile::SetMark(
	long		howFar,
	short		fromWhere)
{
	return( SetFPos(refNum, fromWhere, howFar) );
}


/******************************************************************************
 GetMark

		Retrieve the position of the mark which determines determines the
		starting point for subsequent read/write operations. File must
		already be open.
 ******************************************************************************/

OSErr	ODataFile::GetMark(
	long	*markPos)
{
	*markPos = 0L;
	return( GetFPos(refNum, markPos) );
}


/**** O P E N / C L O S E   M E T H O D S ****/


/******************************************************************************
 Open

		Open the data fork of a file with the specified access permission
 ******************************************************************************/

OSErr	ODataFile::Open(
	SignedByte		permission)
{
	OSErr			errCode;
	
	errCode = HOpen(volNum, dirID, name, permission, &refNum);
	
	if (errCode != noErr) {
		refNum = 0;
	}
	
	return(errCode);
}
	

/******************************************************************************
 Close {OVERRIDE}

		Close a File and make sure contents are written to disk
 ******************************************************************************/

OSErr	ODataFile::Close()
{
	register OSErr		errCode;
	
	if (refNum > 0) {
		errCode = FSClose(refNum);
		if (errCode == noErr) {
			errCode =  FlushVol("\p", volNum);
		}
		refNum = 0;
		return(errCode);
		
	} else {
		return(fnOpnErr);
	}
}


/**** R E A D / W R I T E   M E T H O D S ****/
	

/******************************************************************************
 ReadAll

		Read the entire contents of a file into memory. A newly created
		handle containing the data is returned. File must already be open.
 ******************************************************************************/

OSErr	ODataFile::ReadAll(
	Handle		*contents)
{
	register OSErr	errCode;
	long			length;				/* Length of file in bytes			*/
	
	errCode = GetEOF(refNum, &length);
	if (errCode == noErr) {

		*contents = NewHandleCanFail(length);
		
		if (*contents != NULL) {
			errCode = SetFPos(refNum, fsFromStart, 0L);
			if (errCode == noErr) {
				errCode = FSRead(refNum, &length, **contents);
			}
		}
	}
    return(errCode);
}	


/******************************************************************************
 ReadSome

		Read the specified number of bytes from a file starting at the
		current mark. Space must already have been allocated to store
		the data and a pointer to this space is passed as a parameter.
		The file must already be open.
 ******************************************************************************/

OSErr	ODataFile::ReadSome(
	Ptr			info,
	long		howMuch)
{
	return( FSRead(refNum, &howMuch, info) );
}


/******************************************************************************
 WriteAll

		Write all the data in a handle to a file. The existing contents
		of the file are replaces. File must already be open.
 ******************************************************************************/

OSErr	ODataFile::WriteAll(
	Handle		contents)
{
	register OSErr	errCode;
	long			length;				/* Number of byte to write out		*/
	
	length = GetHandleSize(contents);
	
										/* Move mark to beginning of file	*/
	errCode = SetFPos(refNum, fsFromStart, 0L);
	if (errCode == noErr) {
										/* Write out all the data			*/
		errCode = FSWrite(refNum, &length, *contents);
		if (errCode == noErr) {
										/* Set file length in case previous	*/
										/*   contents was bigger			*/
			errCode = SetEOF(refNum, length);
			if (errCode == noErr) {	
											/* Force write to disk				*/
				errCode = FlushVol("\p", volNum);
			}
		}
	}
    return(errCode); 
}


/******************************************************************************
 WriteSome

		Write the specified number of bytes to a file starting at the
		current mark. The file must already be open.
 ******************************************************************************/

OSErr	ODataFile::WriteSome(
	Ptr			info,
	long		howMuch)
{
	return( FSWrite(refNum, &howMuch, info) );
}
