/******************************************************************************
 CDecorator.c

							The Decorator Class
							
	A Decorator controls the placement and arrangement of windows on the
	screen. Normally, there is only one instance of CDecorator, which is
	stored as a global variable.
	
	SUPERCLASS = CObject
	
	Copyright  1989 Symantec Corporation. All rights reserved.
	
	TCL 1.1 CHANGES
	[
		- modified CenterWindow to place modal windows in the top 1/3 
		  of the main screen.
		- implemented StaggerWindow method. PlaceNewWindow now calls
	      StaggerWindow.
	]

 ******************************************************************************/

#include "Global.h"
#include "Constants.h"
#include "CDecorator.h"
#include "CWindow.h"

#include <Script.h>

/******************************************************************************
 IDecorator

		Initialize a Decorator object
 ******************************************************************************/

void	CDecorator::IDecorator()
{
	wCount = 0;
	index = 1;
	hLocation = LEFT_SMARGIN;
	vLocation = TOP_SMARGIN + GetMBarHeight();
	
	wWidth = screenBits.bounds.right - screenBits.bounds.left
				- hLocation - RIGHT_SMARGIN
				- (MAX_WOFFSETS - 1) * HORIZ_WOFFSET;
				
	wHeight = screenBits.bounds.bottom - screenBits.bounds.top
				- vLocation - BOTTOM_SMARGIN
				- (MAX_WOFFSETS - 1) * VERT_WOFFSET;
}


/******************************************************************************
 StaggerWindow

		Position a new window on the screen, offset in location from
		the previous new window
 ******************************************************************************/

void	CDecorator::StaggerWindow( CWindow *theWindow)
{
	theWindow->Move(hLocation, vLocation);
	
	wCount++;
	if (index == MAX_WOFFSETS) {
		index = 1;
		hLocation = LEFT_SMARGIN;
		vLocation = TOP_SMARGIN + GetMBarHeight();
		
	} else {
		index++;
		hLocation += HORIZ_WOFFSET;
		vLocation += VERT_WOFFSET;
	}

}

/******************************************************************************
 PlaceNewWindow

		Position a new window on the screen, offset in location from
		the previous new window. Also resizes the window to the main screen
		size.
 ******************************************************************************/

void	CDecorator::PlaceNewWindow(
	CWindow		*theWindow)
{				
	StaggerWindow( theWindow);
	theWindow->ChangeSize(wWidth, wHeight);	
}


/******************************************************************************
 CenterWindow

		Center a window on the main screen.
 ******************************************************************************/

void	CDecorator::CenterWindow(
	CWindow		*theWindow)
{
    
    register WindowPeek macWPeek;       /* Toolbox Window Record            */
	short				hPos;			/* Horizontal location for window	*/
	short				vPos;			/* Vertical location for window		*/
    Rect                structRect;     /* Bounds of window's strucRgn      */
    Point               portTopLeft;    /* Top left of window's portRect    */
    GrafPtr             savePort;       /* Need to save and restore port    */
    short				vDivisor;		/* divisor to use to calculate the  */
    									/* window's vertical position 		*/
    					
    /* normal non-modal windows will be centered both horizontally and	*/
    /* vertically. However, modal windows, e.g. modal dialogs, should 	*/
    /* be placed in the top 1/3 of the main screen.						*/
    
	if (theWindow->IsModal() ||
	   (theWindow->procID >= dBoxProc && theWindow->procID <= movableDBoxProc))
		vDivisor = 3;
	else
		vDivisor = 2;    	
    
    /* Center the structure region of the window on the main    */
    /* screen, which is defined by screenBits.bounds.           */
    
    macWPeek = (WindowPeek) theWindow->GetMacPort();
    
                                        /* strucRgn gives the overall size  */
                                        /* of a window                      */
    if (theWindow->IsVisible()) {
        structRect = (**(macWPeek->strucRgn)).rgnBBox;
    }
    else {
            /* For invisible windows, the strucRgn is always empty.         */
            /* Thus, we have to fool the window manager by moving the       */
            /* window offscreen, showing it (it won't actually get          */
            /* drawn since it's offscreen), copying the bounds of the       */
            /* strucRgn, then hiding the window                             */
            
        theWindow->MoveOffScreen();
        ShowHide( (WindowPtr) macWPeek, TRUE);
        structRect = (**(macWPeek->strucRgn)).rgnBBox;
        ShowHide( (WindowPtr) macWPeek, FALSE);
    }
    
        /* Now we need to get the top left corner of the portRect in        */
        /* global coords, since the strucRgn is in global coords.  We       */
        /* want to know how far the portRect is inset from the strucRgn     */
        /* because the MoveWindow trap operates on the portRect, not the    */
        /* strucRgn.                                                        */
        
    GetPort(&savePort);
    SetPort((WindowPtr) macWPeek);
    portTopLeft = topLeft(((GrafPtr) macWPeek)->portRect);
    LocalToGlobal(&portTopLeft);
    SetPort(savePort);
    
        /* Finally, we have all the necessary information.                  */
        
    hPos = screenBits.bounds.left + portTopLeft.h - structRect.left +
            ( (screenBits.bounds.right - screenBits.bounds.left) -
              (structRect.right - structRect.left) ) / 2;
              
        /* Vertically, we have to account for the menu bar.                 */
        
    vPos = screenBits.bounds.top + GetMBarHeight() / vDivisor +
            portTopLeft.v - structRect.top +
            ( (screenBits.bounds.bottom - screenBits.bounds.top) -
              (structRect.bottom - structRect.top) ) / vDivisor;

	theWindow->Move(hPos, vPos);		/* Place window at designated spot	*/
	
	theWindow->ForceNextPrepare();
}


/******************************************************************************
 GetWCount

		Return the count of the number of new windows placed
 ******************************************************************************/

short	CDecorator::GetWCount()
{
	return(wCount);					/* Return instance variable				*/
}