/******************************************************************************
 CDirectorOwner.c

		Abstract class representing objects that can own Directors. Directors
		are Bureaucrats that manage a window. The Application object and
		Directors themselves are DirectorOwners.
		
	SUPERCLASS = CBureaucrat
	
	Copyright  1991 Symantec Corporation. All rights reserved.
	

 ******************************************************************************/

#include "CDirectorOwner.h"
#include "CDirector.h"
#include "CList.h"

/******************************************************************************
 IDirectorOwner
 
 	Initialize a DirectorOwner. 
******************************************************************************/

void CDirectorOwner::IDirectorOwner( CDirectorOwner *aSupervisor)
{
	CBureaucrat::IBureaucrat( aSupervisor);
	itsDirectors = NULL;

}	/* CDirectorOwner::IDirectorOwner */

/******************************************************************************
 AddDirector
 
 	Add a new Director to the director list.
******************************************************************************/

void CDirectorOwner::AddDirector( CDirector *aDirector)
{
	if (!itsDirectors)
	{
		itsDirectors = new( CList);
		itsDirectors->IList();
	}
	itsDirectors->Add( aDirector);

}	/* CDirectorOwner::AddDirector */

/******************************************************************************
 RemoveDirector
 
 	Remove a Director from the director list.
******************************************************************************/

void CDirectorOwner::RemoveDirector( CDirector *aDirector)
{
	if (itsDirectors)
	{
		itsDirectors->Remove( aDirector);
		if (itsDirectors->IsEmpty())
			ForgetObject( itsDirectors);
	}

}	/* CDirectorOwner::RemoveDirector */

/******************************************************************************
 ActivateDirector
 
 	Receive notification that an owned director has been
 	activated.
******************************************************************************/

void CDirectorOwner::ActivateDirector( CDirector *aDirector)
{
	if (itsDirectors)
		itsDirectors->BringFront( aDirector);
	active = TRUE;
	

}	/* CDirectorOwner::ActivateDirector */

/******************************************************************************
 DeactivateDirector
 
 	Receive notification that an owned director has been deactivated.
******************************************************************************/

void CDirectorOwner::DeactivateDirector( CDirector *aDirector)
{
	active = FALSE;

}	/* CDirectorOwner::DeactivateDirector */

/******************************************************************************
 Suspend
 
 	Notify all owned directors that they are being suspended.
******************************************************************************/

		/*
		 *	Send Suspend message to a director
		 */
			static void	Director_Suspend(
				CDirector	*theDirector)
			{
			theDirector->Suspend();
			}

void CDirectorOwner::Suspend( void)
{
								/* Tell all our directors that we	*/
								/* are about to lose control		*/
	if (itsDirectors)
		itsDirectors->DoForEach((EachFunc) Director_Suspend);

}	/* CDirectorOwner::Suspend */

/******************************************************************************
 Resume
 
  	Notify all owned directors that they are being resumed.
******************************************************************************/

		/*
		 *	Send Resume message to a director
		 */
			static void	Director_Resume(
				CDirector	*theDirector)
			{
				theDirector->Resume();
			}

void CDirectorOwner::Resume( void)
{
								/* We are no longer in background	*/
								/* Tell all directors we are back	*/
								/*   in control						*/
	if (itsDirectors)
		itsDirectors->DoForEach((EachFunc) Director_Resume);

}	/* CDirectorOwner::Resume */

/******************************************************************************
 Quit
 
 	Respond to Quit message by closing all its directors.
******************************************************************************/

Boolean CDirectorOwner::Quit( void)
{
	return Close( TRUE);
	
}	/* CDirectorOwner::Quit */

/******************************************************************************
 Close
 
 	Attempt to close all the owned directors. If any directors returns
 	FALSE from the Close message, then the Close is aborted.
******************************************************************************/

Boolean CDirectorOwner::Close( Boolean fQuitting)
{
	if (itsDirectors)
	{
		CDirector *theDirector;
		
		theDirector = (CDirector*) itsDirectors->FirstItem();
		while (theDirector)
		{	
			if (!theDirector->Close( fQuitting))
				return FALSE;
				
			if (!itsDirectors)	// removing last director might dispose
				break;			// the list
			
			theDirector = (CDirector*) itsDirectors->FirstItem();
		}
	}
	
	return TRUE;

}	/* CDirectorOwner::Close */

/******************************************************************************
 Dispose
 
 	Dispose of the DirectorOwner and all its directors.
******************************************************************************/

void CDirectorOwner::Dispose( void)
{
	if (itsDirectors)
	{
		itsDirectors->DisposeAll();
		itsDirectors = NULL;
	}
		
	inherited::Dispose();

}	/* CDirectorOwner::Dispose */