/******************************************************************************
 CError.c

							The Error Class
		
	Error handling class for reporting errors.
	
	SUPERCLASS = CObject
	
	Copyright  1989 Symantec Corporation. All rights reserved.

 ******************************************************************************/

#include "Global.h"
#include "Constants.h"
#include "TBUtilities.h"
#include "CError.h"
#include "CApplication.h"

#include "CEditText.h"
#include <Packages.h>


/**** Global Variables ***/

extern CApplication	*gApplication;		/* Application object				*/
extern CError		*gError;			/* Error handler					*/


extern long         gSleepTime;         /* Max time between events          */


/*=============================================================================
 GrowZoneFunc
 
		Called by the System when a memory request can't be filled. Send
		GrowMemory message to the Application. The bracketing calls to
		the Toolbox traps SetUpA5 and RestoreA5 are required because we
		don't know if A5 is set up properly. See TechNote #136.
 =============================================================================*/

pascal long		GrowZoneFunc(
	Size			bytesNeeded)
{
    
	long				result;			/* Success or failure code			*/
	Handle				dontMove;		/* Block which we must not move		*/
	SignedByte			saveHState;		/* Original attributes of dontMove	*/
	
    long                oldA5 = SetCurrentA5();
    
	dontMove = GZSaveHnd();				/* There may be a memory block		*/
	if (dontMove != NULL) {				/*   which we must not move			*/
		saveHState = HGetState(dontMove);
		HLock(dontMove);				/* If so, lock it down				*/
	}
	
	result = gApplication->GrowMemory(bytesNeeded);
	
	if (dontMove != NULL) {				/* Restore original state of block	*/
		HSetState(dontMove, saveHState);
	}
    
    SetA5(oldA5);

	return(result);
}


/*=============================================================================
 CheckResource

		Check for a NULL resource handle, which indicates that a resource
		could not be found or read into memory. If handle is NULL, post
		a severe error alert.
 =============================================================================*/

void	CheckResource(
	Handle		r)						/* Any kind of resource handle		*/
{
	if (r == NULL) {
		gError->SevereMacError(resNotFound);
	}
}


/*=============================================================================
 CheckAllocation

		Check for a NULL pointer or handle, which indicates that a
		requested memory block could not be allocated.
 =============================================================================*/

void	CheckAllocation(
	void		*p)						/* Any kind of pointer				*/
{
	if (p == NULL) {
		gError->SevereMacError(MemError());
	}
}


/******************************************************************************
 SevereMacError

		Report a Macintosh Operating System Error by posting an Alert
		Box which gives the user the choice of continuing at his/her own
		risk or aborting the program.
 ******************************************************************************/

void	CError::SevereMacError(
	OSErr		macErr)
{
	register Handle		errMsg;			/* Text of error message			*/
	Str255				errNum;			/* Error number as a string			*/

    
    Str255              errorString;
	
										/* Get text for error message		*/
	errMsg = GetResource(ErrMsg_Res, macErr);
	if (errMsg == NULL) {				/* Use default message if a custom	*/
										/*   one is not available			*/
		errMsg = (Handle) GetString(STRosError);
	}
    
    
    if (errMsg == NULL)
    
        if (macErr == resNotFound)
            MissingResources();
            
        else {
            CopyPString((StringPtr)"\pMac OS Error encountered", errorString);
            errMsg = NewHandle(errorString[0] + 1L);
            CopyPString(errorString, (StringPtr) *errMsg);
        }
										
	NumToString((long)macErr, errNum);	/* Convert error number to a string	*/
										
										/* Stuff strings into Alert Box and	*/
										/*   display it						*/
	HLock(errMsg);
	ParamText( (StringPtr) *errMsg, errNum, NULL, NULL);
	HUnlock(errMsg);
	PositionDialog('ALRT', ALRTsevereErr);
	InitCursor();
	if (StopAlert(ALRTsevereErr, NULL) == Cancel) {
		gApplication->Exit();
		ExitToShell();
	} else {
		HiliteMenu(NOTHING);
		gApplication->JumpToEventLoop();
	}
}


/******************************************************************************
 CheckOSError

		Check for a Macintosh Operating System Error and post an Alert
		if necessary. Return TRUE if everything is OK; Return FALSE
		if an error occurred.
		
		Method looks for text for an error message in an ErrMsg_Res resource
		with the same ID number as the error number. If it doesn't find
		one, it uses the default error message which is stored in an
		'STR ' resource with the same ID as that used for the Alert Box.
 ******************************************************************************/

Boolean		CError::CheckOSError(
	OSErr		macErr)
{
	register Handle		errMsg;			/* Text of error message			*/
	Str255				errNum;			/* Error number as a string			*/
	
	if (macErr == noErr) {				/* There was no error				*/
		return(TRUE);
		
	} else {							/* Error encountered				*/
										/* Get text for error message		*/
		errMsg = GetResource(ErrMsg_Res, macErr);
		if (errMsg == NULL) {			/* Use default message if a custom	*/
										/*   one is not available			*/
			errMsg = (Handle) GetString(STRosError);
		}
										/* Convert error number to a string	*/
		NumToString((long)macErr, errNum);
										
										/* Stuff strings into Alert Box and	*/
										/*   display it						*/
		HLock(errMsg);
		ParamText( (StringPtr) *errMsg, errNum, NULL, NULL);
		HUnlock(errMsg);
		PositionDialog('ALRT', ALRTosError);
		InitCursor();
		StopAlert(ALRTosError, NULL);
		return(FALSE);					/* Signal that an error occurred	*/
	}
}


/******************************************************************************
 PostAlert

		Post a standard Mac Alert box with a text string specified by
		a string list resource ID (STR#) and index number
 ******************************************************************************/

void	CError::PostAlert(
	short		STRid,					/* STR# resource ID					*/
	short		index)					/* Index into string list resource	*/
{
	Str255		message;
	
	GetIndString(message, STRid, index);
	ParamText(message, NULL, NULL, NULL);
	PositionDialog('ALRT', ALRTgeneral);
	InitCursor();
	Alert(ALRTgeneral, NULL);
}


/******************************************************************************
 MissingResources

        We tried to load two non-existent resources.  Probably, there is no
        resource file called "<ProjectFileName>.rsrc" in the project folder.
 ******************************************************************************/

void    CError::MissingResources()
{
    typedef struct {            /* Template for a DITL consisting of one    */
        short   numItems;       /* static text item.                        */
        Handle  itemHandle;
        Rect    itemRect;
        short   itemTypeAndLength;  /* Type in high byte, length in low byte.   */
        Str255  itemContent;        /* Actually, this needs to be pure text     */
                                    /* (no length byte).  Also, the number of   */
                                    /* chars must be even.                      */
                                    
    } textItem, *textItemP, **textItemH;            
    
    Str255  theText;
    short   theTextLength,
            TextTopLeft = 20,
            DialogRight = 255,
            DialogBottom = 115;
    long    textItemType = (statText + itemDisable) * 0x0100;
    
    textItemP   theDialogTextP;
    textItemH   theDialogText;
    DialogPtr   theDialog;
    Rect        theDialogRect;
    Boolean     eventReceived;
    EventRecord macEvent;
    
    CopyPString((StringPtr) "\pome resources were not found.  Is there a resource file named '<ProjectFileName>.rsrc' in your project folder? ",
                theText);
    theTextLength = theText[0] + 1L;
    
    theDialogText = (textItemH) NewHandle(sizeof(textItem));
    HLock( (Handle) theDialogText);
    theDialogTextP = *theDialogText;
    
    theDialogTextP->numItems = 0;       /* To indicate one item.    */
    theDialogTextP->itemHandle = NULL;  /* Just a placeholder.      */
    
    SetRect(&theDialogTextP->itemRect, 
            TextTopLeft, TextTopLeft, DialogRight, DialogBottom);
    
    theDialogTextP->itemTypeAndLength = textItemType + theTextLength;
    CopyPString(theText, theDialogTextP->itemContent);
    
    theDialogTextP->itemContent[0] = 'S';   /* Replace length byte with 'S' */
    
    theDialogTextP->itemContent[30] = CARRIAGE_RETURN;  /* Replace these    */
    theDialogTextP->itemContent[31] = CARRIAGE_RETURN;  /* spaces with CRs  */
    theDialogTextP->itemContent[62] = CARRIAGE_RETURN;
    theDialogTextP->itemContent[95] = CARRIAGE_RETURN;
    
    SetRect(&theDialogRect, 0, 0, DialogRight, DialogBottom);
    
    /* Center the dialog in the upper portion of the screen. */
    
    theDialogRect.left = (screenBits.bounds.right - 
                            (theDialogRect.right - theDialogRect.left)) / 2;
    theDialogRect.top = (screenBits.bounds.bottom - 
                            (theDialogRect.bottom - theDialogRect.top)) / 3;
                            
    if (theDialogRect.top < (GetMBarHeight() + 7))
        theDialogRect.top = GetMBarHeight() + 7;
        
    theDialogRect.right += theDialogRect.left;  /* left and top were both 0 */
    theDialogRect.bottom += theDialogRect.top;
    
    theDialog = NewDialog(NULL, &theDialogRect, (StringPtr)"", TRUE, dBoxProc, (WindowPtr) -1L, FALSE,
                          0, (Handle) theDialogText);
    SysBeep(1);
    DrawDialog(theDialog);
    
    /* Dismiss dialog on keyDown or mouseDown */
    
    do  {
            SystemTask();
            eventReceived = GetOSEvent(mDownMask + keyDownMask, &macEvent);
        }
    while (eventReceived == FALSE);
    
    DisposDialog(theDialog);
    gApplication->Exit();
    ExitToShell();
}
