/******************************************************************************
 CFile.c

							The File Class
		
	Class for a disk-based file. Information can be read from or written to
	a File.
	
	SUPERCLASS = CObject
	
	Copyright  1989 Symantec Corporation. All rights reserved.
	
	TCL 1.1 CHANGES
	[
		- changed implementation to use exceptions. If you need or prefer the
			old version, use the compatibility class BFile.
		- implemented ExistsOnDisk, GetMacFileInfo, SpecifyFSSpec, GetFSSpec,
		  and ResolveFileAlias methods
	]

 ******************************************************************************/

#include "Global.h"
#include "CFile.h"
#include "TBUtilities.h"

#include <Aliases.h>


/**** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S ****/


/******************************************************************************
 IFile

		Initialize a File object
 ******************************************************************************/

void	CFile::IFile()
{
	name[0] = 0;						/* Clear instance variables			*/
	volNum = 0;
	dirID = 0L;
}
	

/******************************************************************************
 Dispose {OVERRIDE}

		Dispose of a File object
 ******************************************************************************/

void	CFile::Dispose()
{
	Close();							/* Close Mac File					*/
	
	inherited::Dispose();				/* Pass message on to superclass	*/
}


/******************************************************************************
 Open

		Open a File. Subclasses must override this method.
 ******************************************************************************/

void	CFile::Open(
	SignedByte		permission)
{
}										/* Null Method						*/


/******************************************************************************
 Close

		Close a File. Subclasses must override this method.
 ******************************************************************************/

void	CFile::Close()
{
}										/* Null Method						*/


/**** S P E C I F Y I N G   M E T H O D S ****/


/******************************************************************************
 Specify

		Specify the name and volume number of a File
 ******************************************************************************/

void	CFile::Specify(
	Str63		aName,
	short		aVolNum)
{
	CopyPString(aName, name);
	volNum = aVolNum;
	dirID = 0L;
	if (ExistsOnDisk())
		ResolveFileAlias();
}


/******************************************************************************
 SpecifyFSSpec

		Specify the File using a File Mgr FSSpec record
 ******************************************************************************/

void CFile::SpecifyFSSpec( const FSSpec *aFileSpec)
{
	CopyPString( (ConstStr63Param) aFileSpec->name, name);
	volNum = aFileSpec->vRefNum;
	dirID = aFileSpec->parID;

	if (ExistsOnDisk())
		ResolveFileAlias();
}

/******************************************************************************
 SpecifyHFS

		Specify the name, volume number, and directory ID of a File
 ******************************************************************************/

void	CFile::SpecifyHFS(
	Str63		aName,
	short		aVolNum,
	long		aDirID)
{
	FSSpec	fileSpec;
	
	CopyPString(aName, fileSpec.name);
	fileSpec.vRefNum = aVolNum;
	fileSpec.parID = aDirID;
	
	SpecifyFSSpec( &fileSpec);
}


/******************************************************************************
 SFSpecify

		Specify file parameters from a Standard File reply record
 ******************************************************************************/

void	CFile::SFSpecify(
	SFReply		*macSFReply)
{
	short		aVolNum;
	long		aDirID;
	long		aProcID;
													
	FailOSErr( GetWDInfo(macSFReply->vRefNum, &aVolNum, &aDirID, &aProcID));
	SpecifyHFS(macSFReply->fName, aVolNum, aDirID);
}

/******************************************************************************
 ResolveFileAlias

	Resolve the current File specification in case it is an alias. The
	Finder and Standard File resolve aliases for you, but this may
	be necessary if you open a file directly. The Specify methods
	call ResolveAliasFile for you.
 ******************************************************************************/

void CFile::ResolveFileAlias( void)
{
	FSSpec 	fileSpec;
	Boolean	isFolder, wasAliased;
	
	if (!gSystem.hasAliasMgr) return;
	
	CopyPString( name, fileSpec.name);
	fileSpec.parID = dirID;
	fileSpec.vRefNum = volNum;
	
	FailOSErr( ResolveAliasFile( &fileSpec, TRUE, &isFolder, &wasAliased));
	
	if (wasAliased)	// copy back specification
	{
		CopyPString( fileSpec.name, name);
		dirID = fileSpec.parID;
		volNum = fileSpec.vRefNum;
	}
	
	// raise exception if this is a directory, we only handle files
	
	if (isFolder)
		Failure( paramErr, 0);
}

/**** A C C E S S I N G   M E T H O D S ****/


/******************************************************************************
 GetName

		Retrieve the name of a File
 ******************************************************************************/

void	CFile::GetName(
	Str63		theName)
{
	CopyPString(name, theName);
}

/******************************************************************************
 GetFSSpec

	Fill in a FSSpec record for this File
 ******************************************************************************/

void CFile::GetFSSpec( FSSpec *aFileSpec)
{
	GetName( aFileSpec->name);
	aFileSpec->parID = dirID;
	aFileSpec->vRefNum = volNum;
}

/**** F I L I N G   M E T H O D S ****/


/******************************************************************************
 CreateNew

		Create a new file
 ******************************************************************************/

void	CFile::CreateNew(
	OSType		creator,				/* Creator signature				*/
	OSType		fType)					/* File type						*/
{
	
	FailOSErr( HCreate(volNum, dirID, name, creator, fType));
}


/******************************************************************************
 ThrowOut

		Remove a file from disk, i.e., throw it in the trash.
 ******************************************************************************/

void	CFile::ThrowOut()
{
	Close();							/* Must close file before we can	*/
										/*   throw it out					*/
	FailOSErr( HDelete(volNum, dirID, name) );
}


/******************************************************************************
 ChangeName

		Rename a file. File may or may not be open.
 ******************************************************************************/

void	CFile::ChangeName(
	Str63		newName)
{
	FailOSErr( HRename(volNum, dirID, name, newName));
	
	CopyPString(newName, name);
}

/******************************************************************************
 ExistsOnDisk

		Returns TRUE if file exists on disk matching the current specification
 ******************************************************************************/

Boolean CFile::ExistsOnDisk( void)
{
	FInfo	fileInfo;
	OSErr	err;
	
	err = HGetFInfo( volNum, dirID, name, &fileInfo);
	return err == noErr;

}

/******************************************************************************
 GetMacFileInfo

		Return a File Manager FInfo structure for a file.
 ******************************************************************************/

void CFile::GetMacFileInfo( FInfo *fileInfo)
{
	
	FailOSErr( HGetFInfo( volNum, dirID, name, fileInfo));
}
