/******************************************************************************
 CView.c

							The View Class
		
		Abstract class for objects which have a visual representation
		on the screen.
	
	SUPERCLASS = CBureaucrat
	
	Copyright  1989 Symantec Corporation. All rights reserved.

	TCL 1.1 CHANGES
	[
		- implemented GetWantsClicks method
		- implemented SetCanBeGopher and GetCanBeGopher methods
		- implemented SetID, GetID, and FindByID methods.
		- added cPreparedView class variable.
		- implemented interface changes to GetFrame, GetInterior, GetAperture,
		  and FrameToGlobalR to use LongRects.
		- implemented UseLongCoordinates and ForceNextPrepare methods
		- see comments for DoClick for some important notes
	]
 ******************************************************************************/

#include "Global.h"
#include "CView.h"
#include "TBUtilities.h"
#include "Balloons.h"
#include "CList.h"
#include "TCLUtilities.h"
#include "Constants.h"

/**** Global Variables ****/

extern CBureaucrat	*gGopher;			/* First in line to get commands	*/
extern EventRecord	gLastMouseDown;		/* Previous mouse down event		*/
extern EventRecord	gLastMouseUp;		/* Previous mouse up event			*/
extern CView		*gLastViewHit;		/* Last view clicked in				*/
extern short		gClicks;			/* Click counter, = 1 single click	*/
										/*                = 2 double click	*/
										/*    etc.							*/
extern RgnHandle	gUtilRgn;			/* Utility region					*/

/**** Class Variables *****/

CView	*CView::cPreparedView = NULL;
CView 	*CView::cLastHelpView = NULL;
CView 	*CView::cCurrHelpView = NULL;

/******************************************************************************
 IView

		Initialize a View object. Views initially have no subviews, and
		are invisible and inactive.
 ******************************************************************************/

void	CView::IView(
	CView		*anEnclosure,			/* Enclosing display of this view	*/
	CBureaucrat	*aSupervisor)			/* Supervisor in the command chain	*/
{
										/* Initialize superclass			*/
	CBureaucrat::IBureaucrat(aSupervisor);
	macPort = NULL;						/* Subclass should set the port		*/
	itsEnclosure = anEnclosure;			/* Store enclosing view				*/
	itsSubviews = NULL;

	visible = FALSE;					/* By default, views are invisible,	*/
	active = FALSE;						/*   inactive, and					*/
	wantsClicks = FALSE;				/*   don't handle mouse clicks		*/
	canBeGopher = FALSE;				/*   and don't want to be gGopher   */
	
	ID = 0L;							/* default ID is zero. It is up to	*/
										/* the programmer to provide IDs	*/
										/* and guarantee uniqueness.		*/
	usingLongCoord = FALSE;
	helpResIndex = 0;
}


/******************************************************************************
 IViewRes

		Initialize a View from a resource
 ******************************************************************************/

void	CView::IViewRes(
	ResType		rType,					/* Resource Type of template		*/
	short		resID,					/* Resource ID of template			*/
	CView		*anEnclosure,			/* Enclosing display of this view	*/
	CBureaucrat	*aSupervisor)			/* Supervisor in the command chain	*/
{
	Handle			hRes;				/* Handle to resource template		*/
	
	canBeGopher = FALSE;
	ID = 0L;
	usingLongCoord = FALSE;
	helpResIndex = 0;

	hRes = GetResource(rType, resID);	/* Load resource into memory		*/

	FailNILRes( hRes);					/* Make sure resource exists		*/
	HLock(hRes);						/* Lock resource in heap			*/
										/* Initialize View from template	*/
	IViewTemp(anEnclosure, aSupervisor, *hRes);
	HUnlock(hRes);						/* Unlock resource					*/

}


/******************************************************************************
 IViewTemp

		Initialize a View from a template
 ******************************************************************************/

void	CView::IViewTemp(
	CView		*anEnclosure,			/* Enclosing display of this view	*/
	CBureaucrat	*aSupervisor,			/* Supervisor in the command chain	*/
	Ptr			viewData)				/* Template with View data			*/
{
	register ViewTempP	p;
										/* Initialize superclass			*/
	CBureaucrat::IBureaucrat(aSupervisor);
	macPort = NULL;						/* Subclass should set the port		*/
	itsEnclosure = anEnclosure;			/* Store enclosing view				*/
	itsSubviews = NULL;

	p = (ViewTempP) viewData;			/* Set instance vars from template	*/
	visible = p->visible != 0;
	active = p->active != 0;
	wantsClicks = p->wantsClicks != 0;
	
	usingLongCoord = FALSE;
}


/******************************************************************************
 Dispose

		Dispose of a view and all its subviews
 ******************************************************************************/

void	CView::Dispose()
{
	if (itsSubviews != NULL) {			/* If this view has subviews ...	*/
		itsSubviews->DisposeAll();		/*   Throw them out					*/
		itsSubviews = NULL;
	}
	
	
	if (itsEnclosure != NULL) {			/* Remove ourself from enclosure	*/
		itsEnclosure->RemoveSubview(this);
	}

	if (this == gLastViewHit) {			/* Don't leave a dangling pointer	*/
		gLastViewHit = NULL;
	}

	inherited::Dispose();				/* Pass message on to superclass	*/
}


/******************************************************************************
 Show

		Make a view visible
 ******************************************************************************/

void	CView::Show()
{
	visible = TRUE;
	ForceNextPrepare();
}


/******************************************************************************
 Hide

		Make a view invisible
 ******************************************************************************/

void	CView::Hide()
{
	visible = FALSE;
	if (gGopher == this) {
		itsSupervisor->BecomeGopher( TRUE);
	}
	ForceNextPrepare();
}
	

/******************************************************************************
 Activate

		Make a view active. Also activate all its subviews.
 ******************************************************************************/

		/*
		 *	Activate a View
		 */
			static void	View_Activate(
				CView		*theView)
			{
										/* Make sure that View was active	*/
										/*   and visible BEFORE its			*/
										/*   enclosure was deactivate		*/ 
				if (theView->active && theView->visible) {
										/* Clear flag in case the Activate	*/
										/*   method checks it				*/
					theView->active = FALSE;
					theView->Activate();
				}
			}

void	CView::Activate()
{
	active = TRUE;						/* Set active flag					*/
	ForceNextPrepare();
	if (itsSubviews != NULL) {
		itsSubviews->DoForEach((EachFunc) View_Activate);
	}
}	


/******************************************************************************
 Deactivate

		Make a view inactive
 ******************************************************************************/

		/*
		 *	Deactivate a View
		 */
			static void	View_Deactivate(
				CView		*theView)
			{
				Boolean		saveActive;
				
				if (theView->visible) {
										/* We must save, then restore the	*/
										/*   active flag so we can put		*/
										/*   things back the way they were	*/
										/*   upon a subsequent Activate.	*/
					saveActive = theView->active;
					theView->Deactivate();
					theView->active = saveActive;
				}
			}

void	CView::Deactivate()
{
	active = FALSE;						/* Clear active flag				*/
	ForceNextPrepare();
	if (itsSubviews != NULL) {
		itsSubviews->DoForEach((EachFunc) View_Deactivate);
	}
	if (gGopher == this) {
		itsSupervisor->BecomeGopher( TRUE);
	}
}	


/******************************************************************************
 IsVisible

		Check whether a view is visible
 ******************************************************************************/

Boolean	CView::IsVisible()
{
	return(this->visible);				/* Return instance variable			*/
}
	

/******************************************************************************
 IsActive

		Check whether a view is active. A view is active if its active
		flag is set AND its enclosure is active.
 ******************************************************************************/

Boolean	CView::IsActive()
{
	if (active) {
		if (itsEnclosure != NULL) {
			return(itsEnclosure->IsActive());
		} else {
			return(TRUE);
		}
	} else {
		return(FALSE);
	}
}
	

/******************************************************************************
 ReallyVisible

		Check whether a view and all its enclosures are visible.
 ******************************************************************************/

Boolean	CView::ReallyVisible()
{
	if (visible) {
		return(itsEnclosure->ReallyVisible());
	} else {
		return(FALSE);
	}
}


/******************************************************************************
 DispatchClick

		Mouse was pressed somewhere in the view. Find out if a subview
		which wants to handle mouse clicks was hit. If not, send
		message to ourself to handle the click.
		
		NOTE: The where field of macEvent is in Window coordinates upon entry.
 ******************************************************************************/

void	CView::DispatchClick(
	register EventRecord	*macEvent)	/* Mouse down event record			*/
{
	CView			*hitView;			/* View in which mouse was clicked	*/
	Point			hitPt;				/* In Frame coords of the view hit	*/
	long			hOff;
	long			vOff;
	
	hitView = FindSubview(macEvent->where);
	if (hitView == NULL) {
		CountClicks(this, macEvent);
		
		if (canBeGopher && (gGopher != this))
		{
				// if old gopher refuses to resign, then
				// we can't process this click
				
			if (!BecomeGopher( TRUE))
				return;
		}
		Prepare();
		hitPt = macEvent->where;
		
		if (!usingLongCoord)
			GetOrigin(&hOff, &vOff);
		else
		{
			hOff = thePort->portRect.left;
			vOff = thePort->portRect.top;
		}
		hitPt.h += hOff;
		hitPt.v += vOff;

		DoClick(hitPt, macEvent->modifiers, macEvent->when);
	} else {
		hitView->DispatchClick(macEvent);
	}
}


/******************************************************************************
 DoClick

		Respond to a mouse click within the view. Default action is
		to do nothing.
		
		TCL 1.1
			- When usingLongCoord is FALSE, hitPt is in Frame coordinates,
			  and panes may draw directly in Frame coordinates.
			  
			- When usingLongCoord is TRUE, hitPt is in QuickDraw coordinates,
			  and drawing must be done in QuickDraw coordinates.
			  Use QDToFrame/QDToFrameR to convert from QuickDraw to
			  Frame coordinates, and use FrameToQD/FrameToQDR to convert
			  from Frame to QuickDraw coordinates. These utilities may
			  also be used when usingLongCoord is FALSE, and they still
			  do the right thing.
			  
			- In CView::DispatchClick, if the view's canBeGopher instance 
			  variable is TRUE, the view is sent BecomeGopher, then Prepare, 
			  followed by DoClick, otherwise, only Prepare and DoClick are sent.
			  Therefore in DoClick the view is already the gopher if canBeGopher
			  is TRUE, and the view is already Prepared.

 ******************************************************************************/

void	CView::DoClick(
	Point		hitPt,
	short		modifierKeys,
	long		when)
{
}										/* Null Method						*/
	

/******************************************************************************
 DoMouseUp

		Respond to the mouse button being released after a click within
		the view. Default action is to do nothing.
 ******************************************************************************/

void	CView::DoMouseUp(
	EventRecord		*macEvent)			/* Mouse up event record			*/
{
}										/* Null Method						*/	


/******************************************************************************
 DispatchCursor

		Mouse is inside view and the cursor shape needs to be adjusted.
		Find out if a subview can set the cursor shape. If not, send
		message to ourself to handle it.
 ******************************************************************************/

		/*
		 *	Find area of View not covered by subviews
		 */
			static void	View_SubFrame(
				CView		*theView,
				RgnHandle	uncovered)
			{
				LongRect		frameR;
				Rect			globalR;
				
				if (theView->IsVisible()  &&  theView->wantsClicks) {
					theView->GetFrame(&frameR);
					theView->FrameToGlobalR(&frameR, &globalR);
					RectRgn(gUtilRgn, &globalR);
					DiffRgn(uncovered, gUtilRgn, uncovered);
				}
			}
			
void	CView::DispatchCursor(
	Point		where,					/* Mouse location in Window coords	*/
	RgnHandle	mouseRgn)				/* Region containing the mouse		*/
{
	CView			*hitView;			/* View containing the mouse		*/
	LongRect		frameAperture;
	Rect			globalAperture;
	GrafPtr			savePort;
	Point			saveOrigin;
	Point			offset;
	
	hitView = FindSubview(where);		/* See if mouse is within a subview	*/
	
	if (hitView == NULL) {				/* Mouse not within a subview		*/
	
										/* Calculate the uncovered area of	*/
										/*   this View.						*/
	
		GetAperture( &frameAperture);	/* Start off with the aperture		*/
										/*   in global coordinates			*/
		FrameToGlobalR( &frameAperture, &globalAperture);
		RectRgn(mouseRgn, &globalAperture);
		
		if (itsSubviews != NULL) {		/* Substract the area covered by	*/
										/*   the frames of its subviews		*/
			itsSubviews->DoForEach1((EachFunc1) View_SubFrame, (long)mouseRgn);
		}
		
				/* Now take intersection with the visRgn of View's		*/
				/* port. There could be something, such as a window,	*/
				/* floating above the View. Unfortunately, the visRgn	*/
				/* is the local coordinates and we need it to be in		*/
				/* global coords. Rather than making a copy and			*/
				/* offsetting to global coords, we temporily shift the	*/
				/* View's port so that its local coords are the same as	*/
				/* global coords. This saves us the overhead of making	*/
				/* a copy of the visRgn, which may take up a lot of		*/
				/* memory, at the expense of having to save and restore	*/
				/* the port and origin.									*/
										
		GetPort(&savePort);				/* Save port and origin				*/
		SetPort(macPort);
				
		offset = saveOrigin = topLeft(macPort->portRect);
		
		LocalToGlobal(&offset);
		SetOrigin(offset.h, offset.v);
				  						/* visRgn is in Global coords now	*/
		SectRgn(macPort->visRgn, mouseRgn, mouseRgn);
		
										/* Restore origin and port			*/
		SetOrigin(saveOrigin.h, saveOrigin.v);
		SetPort(savePort);
		
		AdjustCursor(where, mouseRgn);	/* Set cursor shape and possibly	*/
										/*   further restrict the mouseRgn	*/																				
										
	} else {							/* Mouse is within a subview		*/
		hitView->DispatchCursor(where, mouseRgn);
	}
	
	// If balloons are on and no one has shown a balloon, then do it now
	// This could either be because the mouse is not in a subview or because
	// no subview did help. This allows a group of panes to share the same help
	// balloon by putting the help info in their enclosing view

	if (gSystem.hasHelpMgr && HMGetBalloons() && !cCurrHelpView)
	{			
		Rect			r;
		HMMessageRecord helpData;
		Point	  		globalMouse;
		Ptr				tipProc = NULL;
		short			bProc, bVariant, bMethod;
	
		helpData.hmmHelpType = 0;
		GetBalloonInfo(  &helpData, &globalMouse, 
					&r, &tipProc, &bProc, &bVariant, &bMethod); 
		
		ShowHelpBalloon( &helpData, globalMouse, &r, tipProc,
				bProc, bVariant, bMethod);
	}
}
	

/******************************************************************************
 AdjustCursor

		Adjust the shape of the cursor according the position of the mouse.
 ******************************************************************************/

void	CView::AdjustCursor(
	Point		where,					/* Mouse location in Window coords	*/
	RgnHandle	mouseRgn)				/* Region containing the mouse		*/
{
	SetCursor(&arrow);					/* Use the standard arrow cursor	*/
}

/******************************************************************************
 GetBalloonInfo

	Setup the parameters used to show a help balloon. This method uses 
	HMGetIndHelpMsg to some of the parameters from a resource and
	uses the aperture to calculate the others.
	
 ******************************************************************************/

void CView::GetBalloonInfo( struct HMMessageRecord *helpData, 
	                           Point *tip, RectPtr alternateRect,
	                           Ptr *tipProc, short *theProc,
	                           short *variant, short *method)
{
	LongRect		lr;
	Rect			r;
	Point	  		pt;
	short			ignoreCount;
	long			balloonOptions; 
	short			helpResID;

	if (helpResIndex > 0)
	{
		// Get the index of the 'hrct' resource to use for this view
		// helpResIndex is the index into the resource
		
		helpResID = GetHelpResID();
		
		// the tip, alternate rect, and count parameters are not used
		
		FailOSErr( HMGetIndHelpMsg( kHMRectListResType, helpResID, helpResIndex,
							kHMEnabledItem, &balloonOptions,
							&pt, &r, theProc,
							variant, helpData, &ignoreCount));

		GetAperture( &lr);
		FrameToGlobalR( &lr, alternateRect);
		
		// set mouse point to center of aperture.
		
		tip->h = alternateRect->left + (alternateRect->right - alternateRect->left)/2;
		tip->v = alternateRect->top + (alternateRect->bottom - alternateRect->top)/2;

		tipProc = NULL;
		
		if (balloonOptions & hmSaveBitsNoWindow)
			*method = kHMSaveBitsNoWindow;
		else if (balloonOptions & hmSaveBitsWindow)
			*method = kHMSaveBitsNoWindow;
		else
			*method = kHMRegularWindow;
	}
}

/******************************************************************************
 ShowHelpBalloon

	Display a Help balloon for this view using the given parameters. The
	parameters are exactly the same as HMShowBalloon's. When this method
	is called, the caller has already verified that the Help Mgr exists
	and balloons are on. 
	The parameters are initialized by the TCL as follows:
	
		helpData: initialized from the a 'hrct' resource in GetBalloonInfo. 
				  The 'hrct' resource is a resource type defined
				  by the Help Manager.
		tip		: is set to the center of the view, in global coordinates
		altRect : aperture of this view in global coordinates
		tipProc : NULL
		theProc : 0 - the standard ballon definition function
		variant : initialized from helpInfo inst. var - default is 0
		method  : initialized from the 'hrct' resource in GetBalloonInfo
		
	The TCL relies on the altRect parameter being non-zero. When you
	pass a pointer to a Rect in altRect, the Help Mgr will use it to 
	automatically remove the balloon when the cursor leaves the rect. If
	the program passes NULL, then the program must determine when to
	remove the balloon and do so itself. The TCL does not do this for you.
	
 ******************************************************************************/

void CView::ShowHelpBalloon( struct HMMessageRecord *helpData, Point tip, 
							RectPtr altRect, Ptr tipProc, short theProc, 
							short variant, short method)
{
	OSErr	err;
	Boolean savedAlloc;
	
	cCurrHelpView = this;
	
	if ((cLastHelpView != this) && (helpData->hmmHelpType != 0))
	{

		// let this fail if there isn't enough memory
		
		savedAlloc = SetAllocation( kAllocCanFail);
				
		err = HMShowBalloon( helpData, tip, altRect, tipProc, theProc, 
				variant, method);
				
		SetAllocation( savedAlloc);
		
		if (err)
		{
			cCurrHelpView = NULL;
		 	if (err != hmBalloonAborted)
				Failure( err, 0);
		}
	}
}

/******************************************************************************
 GetHelpResID

	Return the resource ID of the 'hrct' resource that has the 
	help balloon info for this view. This method returns the ID of the
	TCL's default 'hrct' resource.
	
 ******************************************************************************/

short CView::GetHelpResID( void)
{
	return kDefaultHelpResID;
}

/******************************************************************************
 GetMacPort

		Return a pointer to the Mac GrafPort of the View
 ******************************************************************************/

GrafPtr		CView::GetMacPort()
{
	return(macPort);					/* Return instance variable			*/
}

	
/******************************************************************************
 GetOrigin

		Return the origin of the view
 ******************************************************************************/

void	CView::GetOrigin(
	long		*theHOrigin,
	long		*theVOrigin)
{
	*theHOrigin = *theVOrigin = 0L;
}

	
/******************************************************************************
 GetFrame

		Return the frame of the view. Subclasses must override.
 ******************************************************************************/

void	CView::GetFrame(
	LongRect		*theFrame)
{
}

	
/******************************************************************************
 GetInterior

		Return the interior of the view. This method assumes that the
		interior is the same as the frame. Subclasses must override this
		method if this is not the case.
 ******************************************************************************/

void	CView::GetInterior(
	LongRect		*theInterior)
{
	GetFrame(theInterior);
}

	
/******************************************************************************
 GetAperture

		Return the portion of the view which is actually visible.
		Subclasses must override this method.
 ******************************************************************************/

void	CView::GetAperture(
	LongRect		*theAperture)
{
}										/* Null Method						*/

/******************************************************************************
 SetID
 
 	Set the ID instance variable for the view. IDs provide a way to
 	find a specific view within the view hierarchy. It is up to the
 	programmer to specify view IDs and to guarantee their uniqueness,
 	if desired. View IDs could be simple numeric constants or character
 	constants, such as 'BTN1'.
******************************************************************************/

void CView::SetID( long anIdentifier)
{
	ID = anIdentifier;
}

/******************************************************************************
 GetID
 
 	Retrieve a view's ID.
******************************************************************************/

long CView::GetID( void)
{
	return ID;
}

/******************************************************************************
 AddSubview

		Add a subview. Check to make sure a subview list exists, and create
		one if it doesn't. This saves a little space because a subview list
		isn't created until the first subview is added. Views without
		subviews don't have to have a reference to an empty List object.
 ******************************************************************************/

void	CView::AddSubview(
	CView		*theSubview)
{
	if (itsSubviews == NULL) {			/* No subview list exists, make one	*/ 
		itsSubviews = new(CList);
		itsSubviews->IList();
	}
	
	itsSubviews->Add(theSubview);
}

	
/******************************************************************************
 RemoveSubview

		Remove a subview
 ******************************************************************************/

void	CView::RemoveSubview(
	CView		*theSubview)
{
	itsSubviews->Remove(theSubview);
}

	
/******************************************************************************
 FindSubview

		Find which subview is hit by a given point and wants clicks. Point
		is specified in window coordinates.
 ******************************************************************************/

		/*
		 *	Check whether a View contains a specified point
		 */
			static Boolean	View_Contains(
				CView		*theView,
				Point		*thePoint)
			{
				if (theView->visible)
					return(theView->Contains(*thePoint));
				else
					return(FALSE);
			}

CView*	CView::FindSubview(
	Point		hitPt)
{
	register CView		*hitSubview;	/* Subview containing hit point		*/

	if (itsSubviews == NULL) {			/* If view has no subview ...		*/
		return(NULL);					/*   No subview can be hit			*/
	
	} else {							/* Search backwards thru subviews	*/
		hitSubview = (CView*) itsSubviews->LastSuccess1((TestFunc1) View_Contains,
														(long)&hitPt);
		if ( (hitSubview == NULL)  ||  !hitSubview->wantsClicks) {
			return(NULL);				/* No subview hit OR the subview	*/
										/*   hit does not process clicks	*/
		} else {
			return(hitSubview);			/* A willing view was hit			*/
		}
	}
}


/******************************************************************************
 MatchView
 
 	Performs a pre-order search for the first view in the view hierarchy
 	that returns TRUE for matchProc. 
 	
 	matchProc is a pointer to a C function of the form:
 		Boolean matchProc( CView *aView, void* matchData);
 		
 	matchData may be any information needed by matchProc, e.g. number,
 	pointer, etc.
 	
******************************************************************************/

	typedef struct tMatchViewInfo
	{
		MatchViewProc 	matchProc;
		void			*matchData;
		CView			*foundView;
		
	} tMatchViewInfo;

	static Boolean View_FindMatch( CObject *aViewObj, void* matchInfoAddr)
	{
		CView *aView = (CView*) aViewObj;
		tMatchViewInfo *matchInfo = (tMatchViewInfo*) matchInfoAddr;
		
		if (matchInfo->matchProc( aView, matchInfo->matchData))
		{
			matchInfo->foundView = aView;
		}
		else if (aView->itsSubviews)
		{
			/* result is returned in matchInfo, just ignore function result */
			(void) (CView*) aView->itsSubviews->FirstSuccess1(
							(TestFunc1) View_FindMatch, (long) matchInfo);
		}
		
		/* searching stops as soon as we find the first match */
		
		return (matchInfo->foundView != NULL);
	}

CView* CView::MatchView( MatchViewProc matchProc, void *matchData)
{
	tMatchViewInfo	matchInfo;
		
	matchInfo.matchProc = matchProc;
	matchInfo.matchData = matchData;
	matchInfo.foundView = NULL;
	
	/* Don't care about return value, the view we want is returned in	*/
	/* matchInfo.foundView or it is NULL								*/
	
	(void) View_FindMatch( this, (void*) &matchInfo);
	
	return matchInfo.foundView;

}	

/******************************************************************************
 FindByID
 
 	Returns the first view in the view hierarchy that has the given ID.
******************************************************************************/


	static Boolean View_FindByID( CView *aView, void* targetID)
	{
		return (aView->GetID() == (long) targetID);
	}
	
CView *CView::FindViewByID( long anID)
{
	return MatchView( View_FindByID, (void*) anID);
}

/******************************************************************************
 Contains

		Check whether a view contains a specified point. Subclasses must
		override this method.
 ******************************************************************************/

Boolean		CView::Contains(
	Point			thePoint)
{
}										/* Null Method						*/

	
/******************************************************************************
 HitSamePart

		Check whether two points hit the same part of the view. The two
		points must actually be within the view (return TRUE from the
		Contain method). The test is used for checking the "close in location"
		requirement for multiple clicks. This means that the same "part" of the
		view is hit by the two points. For text views, a "part" could be a word;
		for graphical views, a "part" could be a graphical object. This
		method assumes that a view has only one part, so all points hit the
		same part. Subclasses which support multiple clicks and have more
		than one part must override this method.
 ******************************************************************************/

Boolean		CView::HitSamePart(
	Point			pointA,
	Point			pointB)
{
	return(TRUE);
}

	
/******************************************************************************
 SubpaneLocation

		Determine the Window coords corresponding to a point which specifies
		the location of a subview. This method assumes that the View uses
		pixel values to specify subview locations. Subclasses which use a
		unit different from one pixel must override this method.
 ******************************************************************************/

void	CView::SubpaneLocation(
	long		hEncl,					/* Point in View's coordinates		*/
	long		vEncl,
	long		*hLocation,				/* Point in Window coordinates		*/
	long		*vLocation)
{
	long		theHOrigin;				/* Origin of the View				*/
	long		theVOrigin;
	
	GetOrigin(&theHOrigin, &theVOrigin);
	*hLocation = hEncl - theHOrigin;	/* Convert point from View to		*/
	*vLocation = vEncl - theVOrigin;	/*   Window coordinates				*/
}

/******************************************************************************
 Prepare

		Do whatever preparations are necessary for drawing in a view.
		Subclasses must override this method, but should call 
		inherited::Prepare to update cPreparedView. 
		
 ******************************************************************************/

void	CView::Prepare()
{
	cPreparedView = this;
}

/******************************************************************************
 ForceNextPrepare

		Clear cPreparedView to force the next Prepare to do a full Prepare.
 ******************************************************************************/

void CView::ForceNextPrepare( void)
{
	cPreparedView = NULL;
}
	
/******************************************************************************
 FrameToGlobalR

		Convert a rectangle from Frame to Global coordinates
 ******************************************************************************/

void	CView::FrameToGlobalR( LongRect *frameRect, Rect *globalRect)
{
}

	
/******************************************************************************
 SetWantsClicks

		Specify whether or not the View responds to mouse clicks
 ******************************************************************************/

void	CView::SetWantsClicks(
	Boolean		aWantsClicks)
{
	wantsClicks = aWantsClicks;
}

/******************************************************************************
 GetWantsClicks
 
 	Check whether view wants to receive clicks
******************************************************************************/

Boolean CView::GetWantsClicks( void)
{
	return wantsClicks;
}

/******************************************************************************
 SetCanBeGopher
 
 	Specify whither or not the view can become the gopher
******************************************************************************/

void CView::SetCanBeGopher( Boolean fCanBeGopher)
{
	canBeGopher = fCanBeGopher;
}

/******************************************************************************
 CanBeGopher
 
 	Check whether view can become the gopher
******************************************************************************/

Boolean CView::CanBeGopher( void)
{
	return canBeGopher && ReallyVisible();
}

/******************************************************************************
 UseLongCoordinates
 
 	Determine the coordinate mode used for this view. When usingLongCoord is
 	TRUE, the view is using a 32-bit coordinate system. This is really only
 	supported in CPane and subclasses, but of course is most useful in 
 	CPanoramas and subclasses. See comments for CView::DoClick,
 	CPane::Draw, and CPane::DrawAll for more information.
******************************************************************************/

void CView::UseLongCoordinates( Boolean fUsing)
{
	usingLongCoord = fUsing;
	ForceNextPrepare();
}

/*=============================================================================
 CountClicks

		Check if mouse down event is part of a multiple click
 =============================================================================*/

void	CountClicks(
	CView		*hitView,
	EventRecord	*macEvent)
{
										/* To be a multiclick, clicks must:	*/
										/*    Hit the same view			*/
										/*    Occur within a certain time	*/
										/*    Be "close" in space			*/
	if ( (hitView == gLastViewHit)  &&
		 ((macEvent->when - gLastMouseUp.when) < GetDblTime())  &&
		 hitView->HitSamePart(gLastMouseDown.where, macEvent->where) ) {
	
		gClicks++;
		
	} else {							/* Reset click counter				*/
		gClicks = 1;
	}
	
	gLastViewHit = hitView;
}