/******************************************************************************
 CDialogDirector.c

		Abstract class for a director that manages a dialog window.
		The dialog may be modal or modeless. Subclasses are responsible 
		for creating the dialog window and dispatching the commands
		generated by the dialog.
		
	SUPERCLASS = CDirector
	
	Copyright  1991 Symantec Corporation. All rights reserved.
	

 ******************************************************************************/

#include "CDialogDirector.h"
#include "CDialog.h"
#include "CDialogText.h"
#include "CBartender.h"
#include "Commands.h"
#include "CApplication.h"
#include "CDesktop.h"
#include "TCLUtilities.h"

extern CBartender	*gBartender;
extern CApplication	*gApplication;
extern CDesktop		*gDesktop;
extern CBureaucrat	*gGopher;
/******************************************************************************
 IDialogDirector
 
 	Initialize a DialogDirector.
 	
******************************************************************************/

void CDialogDirector::IDialogDirector( CDirectorOwner *aSupervisor)
{
	CDirector::IDirector( aSupervisor);
	
	dismissCmd = cmdNull;
	
}	/* CDialogDirector::IDialogDirector */


/******************************************************************************
 DoCommand {OVERRIDE}
 
 	Execute the standard OK and Cancel commands. OK closes the dialog if
 	it can be successfully validated. Cancel always closes the dialog.
 	
******************************************************************************/

void CDialogDirector::DoCommand( long aCmd)
{
	
	switch (aCmd)
	{
		case cmdOK:
			EndDialog( cmdOK, TRUE);
			break;
			
		case cmdCancel:
			EndDialog( cmdCancel, FALSE);
			break;
			
		default:
			inherited::DoCommand( aCmd);
			break;
	}
	
}	/* CDialogDirector::DoCommand */

/******************************************************************************
 Validate
 
 	Pass the Validate message to the dialog window.
 	
******************************************************************************/

Boolean CDialogDirector::Validate( void)
{
	ASSERT( member( itsWindow, CDialog));
	
	return ((CDialog*) itsWindow)->Validate();

}	/* CDialogDirector::Validate */

/******************************************************************************
 DoModalDialog
 
 	Enters a modal loop until the dialog is dismissed by the user. By default,
 	this only happens when cmdOK or cmdCancel are received, but placing any
 	command into the dismissCmd instance variable will end the modal loop.
 	The dismissCmd is returned as the method result.
 	
******************************************************************************/

long CDialogDirector::DoModalDialog( long defaultCmd)
{
	CDialog *itsDialog = (CDialog*) itsWindow;
	CBureaucrat	*savedGopher;
	
		// Setup the desired default command, and make sure
		// the window is modal and visible.
		
	itsDialog->SetDefaultCmd( defaultCmd);
	itsDialog->SetModal( kModal);
	itsDialog->Select();
	DisableTheMenus();
	
		// Since we just disabled the gopher's menus out from under it,
		// we need to trick it into becoming the gopher again.
		
	savedGopher = gGopher;
	this->BecomeGopher( TRUE);
	savedGopher->BecomeGopher( TRUE);
	
	TRY			/* post an exception handler to prevent exceptions from */
	{			/* prematurely aborting the dialog						*/
		do
		{
			gApplication->Process1Event();
			
		} while( dismissCmd == cmdNull);
		
	}
	CATCH
	{
		if (gLastError != kSilentErr)
			ErrorAlert( gLastError, gLastMessage);
		RETRY;
	}
	ENDTRY;
	
	EnableTheMenus();		

	return dismissCmd;
	
}	/* CDialogDirector::DoModalDialog */

/******************************************************************************
 DisableTheMenus
 
 	Disable all but the edit menu before starting a modal dialog.
 	
******************************************************************************/

void CDialogDirector::DisableTheMenus( void)
{
	gBartender->DisableMenuBar();

}	/* CDialogDirector::DisableTheMenus */

/******************************************************************************
 EnableTheMenus
 
 	Enable all menus following a modal dialog.
 	
******************************************************************************/

void CDialogDirector::EnableTheMenus( void)
{
	gBartender->EnableMenuBar();

}	/* CDialogDirector::EnableTheMenus */

/******************************************************************************
 BeginDialog
 
 	Open a dialog by selecting the window, activating the first gopher, 
 	and if the gopher is a text field, selecting all its text.
 	
******************************************************************************/

void CDialogDirector::BeginDialog( void)
{
	CDialog			*itsDialog = (CDialog*) itsWindow;
	tGopherInfo		gopherInfo;
	
	itsDialog->Select();
	itsDialog->FindGophers( &gopherInfo);
	itsGopher = gopherInfo.firstGopher? gopherInfo.firstGopher : itsDialog;
	
	if (member( itsGopher, CAbstractText))
		((CAbstractText*)itsGopher)->SelectAll( TRUE);
		
	if (active)
		itsGopher->BecomeGopher( TRUE);

}	/* CDialogDirector::BeginDialog */

/******************************************************************************
 EndDialog
 
 	End the dialog and optionally validate it. The window is not closed.
 	
******************************************************************************/

Boolean CDialogDirector::EndDialog( long withCmd, Boolean fValidate)
{	
	Boolean closeIt = TRUE;
	
	if (itsWindow)
	{
		if (fValidate)
			closeIt = Validate();
			
		if (closeIt)
			dismissCmd = withCmd;
	}
	return closeIt;
	
}	/* CDialogDirector::EndDialog */

/******************************************************************************
 Close {OVERRIDE}
 
 	Override the Close method to attempt to validate the dialog before
 	closing it.
 	
******************************************************************************/

Boolean CDialogDirector::Close( Boolean fQuitting)
{	
	if (itsWindow  && !itsWindow->active)
		itsWindow->Select();

	if (EndDialog( cmdOK, TRUE))
		return inherited::Close( fQuitting);
	else
		return FALSE;

}	/* CDialogDirector::Close */