/******************************************************************************
 CResFile.c

							The Resource File Class
		
	SUPERCLASS = CFile
	
	Copyright  1989 Symantec Corporation. All rights reserved.
	
	TCL 1.1 CHANGES
	[
		- changed implementation to use exceptions
		- implemented HasResFork, CreateResFork, Update, and IsOpen methods
	]

 ******************************************************************************/

#include "Global.h"
#include "CResFile.h"


/**** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S ****/


/******************************************************************************
 IResFile

		Initialize a ResFile object
 ******************************************************************************/

void	CResFile::IResFile()
{
	CFile::IFile();

	refNum = 0;
}


/******************************************************************************
 Open {OVERRIDE}

		Open the resource fork of a file with the specified access permission.
		Opening the file automatically makes it the current resource file.
 ******************************************************************************/

void	CResFile::Open(
	SignedByte		permission)
{
	Boolean wasLocked;
	
	ASSERT( IsOpen() == FALSE);

	wasLocked = Lock( TRUE);
	refNum = HOpenResFile(volNum, dirID, name, permission);
	Lock( wasLocked);
	
	if (refNum == -1) {
		refNum = 0;
		FailResError();
	}
}
	

/******************************************************************************
 Close {OVERRIDE}

		Close a File and make sure contents are written to disk
 ******************************************************************************/

void	CResFile::Close()
{
	if (refNum > 0) {
		CloseResFile(refNum);
		refNum = 0;
		FailOSErr( FlushVol( NULL, volNum) );
	}
}
	

/******************************************************************************
 MakeCurrent

		Make this the current resource file
 ******************************************************************************/

void	CResFile::MakeCurrent()
{
	ASSERT( IsOpen());

	UseResFile(refNum);
}

/******************************************************************************
 IsOpen

		Returns TRUE if the resource fork of this file is open.
 ******************************************************************************/

Boolean	CResFile::IsOpen( void)
{
	return refNum != 0;

}	/* CFile::IsOpen */

/******************************************************************************
 Update

	Updates the resource file on disk.
 ******************************************************************************/

void CResFile::Update( void)
{
	ASSERT( IsOpen());
	
	UpdateResFile( refNum);
	FailResError();
	
	FailOSErr( FlushVol( NULL, volNum));

}	/* CResFile::Update */

/******************************************************************************
 HasResFork

		Returns TRUE if the file has a resource fork.
 ******************************************************************************/

Boolean	CResFile::HasResFork( void)
{
	HFileParam	pBlock;
	
	pBlock.ioNamePtr = name;
	pBlock.ioVRefNum = volNum;
	pBlock.ioFDirIndex = 0;
	pBlock.ioDirID = dirID;
	
	FailOSErr( PBHGetFInfo( (HParmBlkPtr) &pBlock, FALSE));
	
	/* Assume that if the resource fork logical EOF is non-zero	*/
	/* then a resource fork exists.								*/
	
	return pBlock.ioFlRLgLen != 0;


}	/* CResFile::HasResFork */

/******************************************************************************
 CreateNew {OVERRIDE}

		Creates the file on disk and a new resource fork.
 ******************************************************************************/

void CResFile::CreateNew(OSType creator, OSType fType)
{
	Boolean wasLocked;
	
	/* only call inherited if file doesn't already exist on disk	*/
	if (!ExistsOnDisk())
		inherited::CreateNew( creator, fType);
	
	wasLocked = Lock(TRUE);
	HCreateResFile( volNum, dirID, name);
	Lock( wasLocked);
	
	FailResError();

}	/* CResFile::CreateNew */


