/******************************************************************************
 CTextStyleTask.c

		
		Provides undo support for font, size, style, alignment, and 
		spacing commands in CAbstractText subclasses.
		
		An instance of this class is automatically created by CAbstract 
		text for formatting operations. CEditText can use this
		class directly. Other subclasses of CAbstractText may need
		to subclass CTextStyleTask. For example, CStyleText subclasses
		CTextStyleTask because it must deal directly with the styled scrap 
		format used by Styled TextEdit.
		
		

	SUPERCLASS = CTask
	
	Copyright  1991 Symantec Corporation. All rights reserved.
	

 ******************************************************************************/

#include "CTextStyleTask.h"
#include "TBUtilities.h"
#include "Commands.h"
#include "CBartender.h"
#include "CAbstractText.h"
#include "Global.h"
#include <Packages.h>

extern CBartender	*gBartender;

/******************************************************************************
 ITextStyleTask
 
 	Initialize the task
******************************************************************************/

void CTextStyleTask::ITextStyleTask( CAbstractText *aTextPane, 
						long aStyleCmd, short taskIndex)
{
	
	CTask::ITask( taskIndex);
	
	itsTextPane = aTextPane;
	styleCmd = aStyleCmd;
	styleAttribute = 0;
	
}	/* CTextStyleTask::ITextStyleTask */


/******************************************************************************
 Dispose
 
 	Dispose of a CTextStyleTask. This method clears the reference that
 	the text pane maintains to this task.
******************************************************************************/

void CTextStyleTask::Dispose( void)
{
	if (itsTextPane)
	{
		if (itsTextPane->itsLastTask == this)
			itsTextPane->itsLastTask = NULL;
	}
	
	inherited::Dispose();
	
}	/* CTextStyleTask::Dispose */

/******************************************************************************
 Do
 
 	Saves the original formatting, then performs the user's formatting command 
 	on the text pane. 
******************************************************************************/

void CTextStyleTask::Do( void)
{
	Str255		itemName;
	long		fontSize;
 		
 	SaveStyle();
 	
	oldAlignCmd = itsTextPane->GetAlignCmd();
	oldSpacingCmd = itsTextPane->GetSpacingCmd();
	
	if (styleCmd < 0) 
	{
	
		switch (HiShort(-styleCmd)) 
		{
		
			case MENUfont:
				gBartender->GetCmdText(styleCmd, itemName);
				itsTextPane->SetFontName( itemName);
				styleAttribute = doFont;
				break;
				
			case MENUsize:
				gBartender->GetCmdText(styleCmd, itemName);
				StringToNum(itemName, &fontSize);
				itsTextPane->SetFontSize( fontSize);
				styleAttribute = doSize;
				break;			
		}
	}
	else
	{
		switch( styleCmd)
		{
			case cmdPlain:
				itsTextPane->SetFontStyle(NOTHING);
				styleAttribute = doFace;
				break;
				
			case cmdBold:					/* Styles are set by flipping bits	*/
			case cmdItalic:					/*   Bold is bit 0, Italic is bit 1	*/
			case cmdUnderline:				/*   Underline is bit 2, etc.		*/
			case cmdOutline:				/*   By cleverly numbering these	*/
			case cmdShadow:					/*   commands, we can just shift	*/
			case cmdCondense:				/*   a bit to get the proper style	*/
			case cmdExtend:
				itsTextPane->SetFontStyle(1 << (styleCmd - cmdBold));
				styleAttribute = doFace;
				break;
				
			case cmdAlignLeft:
			case cmdAlignCenter:
			case cmdAlignRight:
				itsTextPane->SetAlignCmd( styleCmd);
				styleAttribute = doAlign;
				break;
				
			case cmdSingleSpace:
			case cmd1HalfSpace:
			case cmdDoubleSpace:
				itsTextPane->SetSpacingCmd( styleCmd);
				styleAttribute = doSpacing;
				break;
		}
	}

}	/* CTextStyleTask::Do */

/******************************************************************************
 Undo
 
 	This method does both undo and redo. Swaps the previously
 	saved formatting with the new formatting.
 	
******************************************************************************/

void CTextStyleTask::Undo( void)
{
	TextStyle	swapStyle, newStyle;
	
	itsTextPane->Prepare();	
	
	swapStyle = oldStyle;		/* save the current styling		*/
	
	SaveStyle();

	newStyle = oldStyle;
	oldStyle = swapStyle;		/* restore the previous styling */

	RestoreStyle();
	
	oldStyle = newStyle;		/* swap current and previous	*/
		
}	/* CTextStyleTask::Undo */

/******************************************************************************
 SaveStyle
 
 	Save the style of the text. Depending upon the actial implementation
 	of the text pane, this could be the global styling (as in standard 
 	TextEdit) or the styling of the selection.
 	
******************************************************************************/

void CTextStyleTask::SaveStyle( void)
{
	TextStyle	style;
	short		styleAttrib;

	styleAttrib = doAll;
	itsTextPane->GetTextStyle( &styleAttrib, &style);
	oldStyle = style;
}

/******************************************************************************
 RestoreStyle
 
 	Restores the previously saved formatting.
 	
******************************************************************************/

void CTextStyleTask::RestoreStyle( void)
{
	long		alignCmd, spacingCmd; 

	switch (styleAttribute)
	{
		case doFont:
			itsTextPane->SetFontNumber( oldStyle.tsFont);
			break;
			
		case doFace:
			itsTextPane->SetFontStyle(1 << (styleCmd - cmdBold));
			break;
			
		case doSize:
			itsTextPane->SetFontSize( oldStyle.tsSize);
			break;
			
		case doAlign:
			alignCmd = itsTextPane->GetAlignCmd();
			itsTextPane->SetAlignCmd( oldAlignCmd);
			oldAlignCmd = alignCmd;
			break;
			
		case doSpacing:
			spacingCmd = itsTextPane->GetSpacingCmd();
			itsTextPane->SetSpacingCmd( oldSpacingCmd);
			oldSpacingCmd = spacingCmd;
			break;
		}
}